@@ -183,10 +183,7 @@ shishi_ap_set_tktoptionsasn1usage (Shishi_ap * ap,
   int buflen;
   int res;
 
-  buf = malloc (BUFSIZ);
-  buflen = BUFSIZ;
-
-  res = shishi_a2d_field (ap->handle, node, field, buf, &buflen);
+  res = shishi_a2d_new_field (ap->handle, node, field, &buf, &buflen);
   if (res != SHISHI_OK)
     return res;
 
@@ -496,11 +493,14 @@ shishi_ap_req_der_new (Shishi_ap * ap, char **out, int *outlen)
  * Return value: Returns SHISHI_OK.
  **/
 int
-shishi_ap_req_der_set (Shishi_ap * ap, char *der, int derlen)
+shishi_ap_req_der_set (Shishi_ap * ap, char *der, size_t derlen)
 {
   ap->apreq = shishi_der2asn1_apreq (ap->handle, der, derlen);
 
-  return SHISHI_OK;
+  if (ap->apreq)
+    return SHISHI_OK;
+  else
+    return SHISHI_ASN1_ERROR;
 }
 
 /**
@@ -583,6 +583,9 @@ shishi_ap_req_process (Shishi_ap * ap, Shishi_key * key)
   Shishi_key *tktkey;
   int rc;
 
+  if (VERBOSEASN1 (ap->handle))
+    shishi_apreq_print (ap->handle, stdout, ap->apreq);
+
   rc = shishi_apreq_get_ticket (ap->handle, ap->apreq, &ticket);
   if (rc != SHISHI_OK)
     {
@@ -611,6 +614,10 @@ shishi_ap_req_process (Shishi_ap * ap, Shishi_key * key)
       return rc;
     }
 
+  if (VERBOSEASN1 (ap->handle))
+    shishi_encticketpart_print (ap->handle, stdout,
+				shishi_tkt_encticketpart (tkt));
+
   rc = shishi_apreq_decrypt (ap->handle, ap->apreq, tktkey, SHISHI_KEYUSAGE_APREQ_AUTHENTICATOR,	/* XXX */
 			     &authenticator);
   if (rc != SHISHI_OK)
@@ -620,6 +627,9 @@ shishi_ap_req_process (Shishi_ap * ap, Shishi_key * key)
       return rc;
     }
 
+  if (VERBOSEASN1 (ap->handle))
+    shishi_authenticator_print (ap->handle, stdout, ap->authenticator);
+
   ap->tkt = tkt;
   ap->authenticator = authenticator;
 
@@ -688,7 +698,7 @@ shishi_ap_rep_set (Shishi_ap * ap, Shishi_asn1 aprep)
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_ap_rep_der (Shishi_ap * ap, char *out, int *outlen)
+shishi_ap_rep_der (Shishi_ap * ap, char *out, size_t * outlen)
 {
   int rc;
 
@@ -715,7 +725,7 @@ shishi_ap_rep_der (Shishi_ap * ap, char *out, int *outlen)
  * Return value: Returns SHISHI_OK.
  **/
 int
-shishi_ap_rep_der_set (Shishi_ap * ap, char *der, int derlen)
+shishi_ap_rep_der_set (Shishi_ap * ap, char *der, size_t derlen)
 {
   Shishi_asn1 aprep;
 
@@ -841,7 +851,7 @@ shishi_ap_rep_verify (Shishi_ap * ap)
  * error.
  **/
 int
-shishi_ap_rep_verify_der (Shishi_ap * ap, char *der, int derlen)
+shishi_ap_rep_verify_der (Shishi_ap * ap, char *der, size_t derlen)
 {
   int res;
 
