@@ -254,16 +254,13 @@ shishi_aprep_enc_part_set (Shishi * handle,
 			   Shishi_asn1 aprep,
 			   int etype, const char *buf, int buflen)
 {
-  char format[BUFSIZ];
   int res;
 
-  res = shishi_asn1_write (handle, aprep, "enc-part.cipher",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, aprep, "enc-part.cipher", buf, buflen);
   if (res != SHISHI_OK)
     return res;
 
-  sprintf (format, "%d", etype);
-  res = shishi_asn1_write (handle, aprep, "enc-part.etype", format, 0);
+  res = shishi_asn1_write_integer (handle, aprep, "enc-part.etype", etype);
   if (res != SHISHI_OK)
     return res;
 
@@ -277,9 +274,9 @@ shishi_aprep_enc_part_add (Shishi * handle,
 			   Shishi_asn1 encapreppart)
 {
   int res;
-  char buf[BUFSIZ];
+  char *buf;
   size_t buflen;
-  char der[BUFSIZ];
+  char *der;
   int derlen = BUFSIZ;
   Shishi_key *key;
 
@@ -287,7 +284,7 @@ shishi_aprep_enc_part_add (Shishi * handle,
   if (res != SHISHI_OK)
     return res;
 
-  res = shishi_a2d (handle, encapreppart, der, &derlen);
+  res = shishi_new_a2d (handle, encapreppart, &der, &derlen);
   if (res != SHISHI_OK)
     {
       shishi_error_printf (handle, "Could not DER encode authenticator: %s\n",
@@ -301,18 +298,22 @@ shishi_aprep_enc_part_add (Shishi * handle,
       derlen++;
     }
 
-  buflen = BUFSIZ;
   res = shishi_encrypt (handle, key, SHISHI_KEYUSAGE_ENCAPREPPART,
-			der, derlen, buf, &buflen);
+			der, derlen, &buf, &buflen);
+
+  free (der);
+
   if (res != SHISHI_OK)
     {
-      shishi_error_printf (handle, "des_encrypt fail\n");
+      shishi_error_printf (handle, "APRep encryption failed\n");
       return res;
     }
 
   res = shishi_aprep_enc_part_set (handle, aprep, shishi_key_type (key),
 				   buf, buflen);
 
+  free (buf);
+
   return res;
 }
 
@@ -364,11 +365,10 @@ shishi_aprep_enc_part_make (Shishi * handle,
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_aprep_get_enc_part_etype (Shishi * handle, Shishi_asn1 aprep,
-				 int *etype)
+shishi_aprep_get_enc_part_etype (Shishi * handle,
+				 Shishi_asn1 aprep, int32_t * etype)
 {
-  return shishi_asn1_integer_field (handle, aprep, etype,
-				    "enc-part.etype");
+  return shishi_asn1_read_int32 (handle, aprep, "enc-part.etype", etype);
 }
 
 int
@@ -379,10 +379,10 @@ shishi_aprep_decrypt (Shishi * handle,
 {
   int res;
   int i;
-  size_t buflen = BUFSIZ;
-  char buf[BUFSIZ];
+  char *buf;
+  size_t buflen;
   char cipher[BUFSIZ];
-  int cipherlen;
+  size_t cipherlen;
   int etype;
 
   res = shishi_aprep_get_enc_part_etype (handle, aprep, &etype);
@@ -399,13 +399,11 @@ shishi_aprep_decrypt (Shishi * handle,
     return res;
 
   res = shishi_decrypt (handle, key, keyusage, cipher, cipherlen,
-			buf, &buflen);
+			&buf, &buflen);
   if (res != SHISHI_OK)
     {
-      if (VERBOSE (handle))
-	printf ("decrypt failed: %s\n", shishi_strerror_details (handle));
       shishi_error_printf (handle,
-			   "decrypt fail, most likely wrong password\n");
+			   "APRep decryption failed, wrong password?\n");
       return res;
     }
 
