@@ -39,7 +39,6 @@ shishi_authenticator (Shishi * handle)
   Shishi_asn1 node = NULL;
   struct timeval tv;
   struct timezone tz;
-  char usec[BUFSIZ];
 
   node = shishi_asn1_authenticator (handle);
   if (!node)
@@ -267,8 +266,7 @@ shishi_authenticator_set_crealm (Shishi * handle,
 {
   int res;
 
-  res = shishi_asn1_write (handle, authenticator, "crealm",
-			   crealm, 0);
+  res = shishi_asn1_write (handle, authenticator, "crealm", crealm, 0);
   if (res != SHISHI_OK)
     return res;
 
@@ -335,8 +333,7 @@ shishi_authenticator_ctime_get (Shishi * handle,
   int res;
 
   len = GENERALIZEDTIME_TIME_LEN + 1;
-  res = shishi_asn1_field (handle, authenticator,
-			   ctime, &len, "ctime");
+  res = shishi_asn1_field (handle, authenticator, ctime, &len, "ctime");
   if (res == SHISHI_OK && len == GENERALIZEDTIME_TIME_LEN)
     ctime[len] = '\0';
 
@@ -355,8 +352,7 @@ shishi_authenticator_ctime_get (Shishi * handle,
  **/
 int
 shishi_authenticator_ctime_set (Shishi * handle,
-				Shishi_asn1 authenticator,
-				char *ctime)
+				Shishi_asn1 authenticator, char *ctime)
 {
   int res;
 
@@ -380,8 +376,7 @@ shishi_authenticator_ctime_set (Shishi * handle,
  **/
 int
 shishi_authenticator_cusec_get (Shishi * handle,
-				Shishi_asn1 authenticator,
-				int *cusec)
+				Shishi_asn1 authenticator, int *cusec)
 {
   int res;
 
@@ -404,8 +399,7 @@ shishi_authenticator_cusec_get (Shishi * handle,
  **/
 int
 shishi_authenticator_cusec_set (Shishi * handle,
-				Shishi_asn1 authenticator,
-				int cusec)
+				Shishi_asn1 authenticator, int cusec)
 {
   int res;
 
@@ -444,8 +438,7 @@ shishi_authenticator_remove_cksum (Shishi * handle, Shishi_asn1 authenticator)
 
   /* XXX remove this function */
 
-  res = shishi_asn1_write (handle, authenticator, "cksum",
-			   NULL, 0);
+  res = shishi_asn1_write (handle, authenticator, "cksum", NULL, 0);
   if (res != SHISHI_OK)
     return res;
 
@@ -468,12 +461,13 @@ shishi_authenticator_remove_cksum (Shishi * handle, Shishi_asn1 authenticator)
 int
 shishi_authenticator_cksum (Shishi * handle,
 			    Shishi_asn1 authenticator,
-			    int *cksumtype, char *cksum, size_t * cksumlen)
+			    int32_t * cksumtype,
+			    char *cksum, size_t * cksumlen)
 {
   int res;
 
-  res = shishi_asn1_integer_field (handle, authenticator, cksumtype,
-				   "cksum.cksumtype");
+  res = shishi_asn1_read_int32 (handle, authenticator,
+				"cksum.cksumtype", cksumtype);
   if (res != SHISHI_OK)
     return res;
 
@@ -504,19 +498,20 @@ shishi_authenticator_cksum (Shishi * handle,
 int
 shishi_authenticator_set_cksum (Shishi * handle,
 				Shishi_asn1 authenticator,
-				int cksumtype, char *cksum, int cksumlen)
+				int32_t cksumtype,
+				char *cksum, size_t cksumlen)
 {
   int res;
 
-  res = shishi_asn1_write_integer (handle, authenticator,
-				   "cksum.cksumtype", cksumtype);
+  res = shishi_asn1_write_int32 (handle, authenticator,
+				 "cksum.cksumtype", cksumtype);
   if (res != SHISHI_OK)
-    return SHISHI_ASN1_ERROR;
+    return res;
 
   res = shishi_asn1_write (handle, authenticator, "cksum.checksum",
 			   cksum, cksumlen);
   if (res != SHISHI_OK)
-    return SHISHI_ASN1_ERROR;
+    return res;
 
   return SHISHI_OK;
 }
@@ -543,18 +538,18 @@ shishi_authenticator_add_cksum (Shishi * handle,
 
   if (data && datalen > 0)
     {
-      char cksum[BUFSIZ];
+      char *cksum;
       int cksumlen;
       int cksumtype = shishi_cipher_defaultcksumtype (shishi_key_type (key));
 
-      cksumlen = sizeof (cksum);
       res = shishi_checksum (handle, key, keyusage, cksumtype,
-			     data, datalen, cksum, &cksumlen);
+			     data, datalen, &cksum, &cksumlen);
       if (res != SHISHI_OK)
 	return res;
 
       res = shishi_authenticator_set_cksum (handle, authenticator,
 					    cksumtype, cksum, cksumlen);
+      free (cksum);
     }
   else
     res = shishi_authenticator_remove_cksum (handle, authenticator);
@@ -603,8 +598,7 @@ shishi_authenticator_add_authorizationdata (Shishi * handle,
 					    int adtype,
 					    char *addata, int addatalen)
 {
-  char format[BUFSIZ];
-  char buf[BUFSIZ];
+  char *format;
   int res;
   int i;
 
@@ -614,19 +608,21 @@ shishi_authenticator_add_authorizationdata (Shishi * handle,
     return res;
 
   res = shishi_asn1_number_of_elements (handle, authenticator,
-					"authorization-data",
-					&i);
+					"authorization-data", &i);
   if (res != SHISHI_OK)
     return res;
 
-  sprintf (buf, "%d", adtype);
-  sprintf (format, "authorization-data.?%d.ad-type", i);
-  res = shishi_asn1_write (handle, authenticator, format, buf, 0);
+  asprintf (&format, "authorization-data.?%d.ad-type", i);
+  res = shishi_asn1_write_integer (handle, authenticator, format, adtype);
   if (res != SHISHI_OK)
-    return res;
+    {
+      free (format);
+      return res;
+    }
 
   sprintf (format, "authorization-data.?%d.ad-data", i);
   res = shishi_asn1_write (handle, authenticator, format, addata, addatalen);
+  free (format);
   if (res != SHISHI_OK)
     return res;
 
@@ -659,23 +655,20 @@ shishi_authenticator_authorizationdata (Shishi * handle,
   int i;
 
   res = shishi_asn1_number_of_elements (handle, authenticator,
-					"authorization-data",
-					&i);
+					"authorization-data", &i);
   if (res != SHISHI_OK)
     return SHISHI_ASN1_ERROR;
 
   if (nth > i)
     return SHISHI_OUT_OF_RANGE;
 
-  shishi_asprintf (&format, "authorization-data.?%d.ad-type",
-		   nth);
-  res = shishi_asn1_integer_field (handle, authenticator, adtype, format);
+  asprintf (&format, "authorization-data.?%d.ad-type", nth);
+  res = shishi_asn1_read_int32 (handle, authenticator, format, adtype);
   free (format);
   if (res != SHISHI_OK)
     return res;
 
-  shishi_asprintf (&format, "authorization-data.?%d.ad-data",
-		   i);
+  asprintf (&format, "authorization-data.?%d.ad-data", i);
   res = shishi_asn1_field (handle, authenticator, addata, addatalen, format);
   free (format);
   if (res != SHISHI_OK)
