@@ -22,7 +22,16 @@
 /* XXX several functions with out/outlen writes longer than the outlen */
 
 #include "internal.h"
+#ifdef USE_GCRYPT
 #include <gcrypt.h>
+#else
+#include "hmac.h"
+#include "des.h"
+#include "aes.h"
+#include "cbc.h"
+#include "cbc-cts.h"
+#include "cbc-mac.h"
+#endif
 
 static void
 escapeprint (const char *str, int len)
@@ -212,36 +221,54 @@ ocadd (char *add1, char *add2, char *sum, int len)
 static int
 simplified_hmac (Shishi * handle,
 		 Shishi_key * key,
-		 const char *in,
-		 size_t inlen, char *outhash, size_t outhashlen)
+		 const char *in, size_t inlen,
+		 char **outhash, size_t * outhashlen)
 {
-  GCRY_MD_HD mdh;
+#if USE_GCRYPT
+  gcry_md_hd_t mdh;
   int halg = GCRY_MD_SHA1;
   size_t hlen = gcry_md_get_algo_dlen (halg);
   unsigned char *hash;
-  int res;
+  gpg_error_t err;
 
-  mdh = gcry_md_open (halg, GCRY_MD_FLAG_HMAC);
-  if (mdh == NULL)
-    return SHISHI_GCRYPT_ERROR;
-
-  res = gcry_md_setkey (mdh, shishi_key_value (key), shishi_key_length (key));
-  if (res != GCRYERR_SUCCESS)
+  err = gcry_md_open (&mdh, halg, GCRY_MD_FLAG_HMAC);
+  if (err != GPG_ERR_NO_ERROR)
     {
-      shishi_error_set (handle, gcry_strerror (res));
-      return SHISHI_GCRYPT_ERROR;
+      shishi_error_printf (handle, "Libgcrypt md open failed");
+      shishi_error_set (handle, gpg_strerror (err));
+      return SHISHI_CRYPTO_INTERNAL_ERROR;
     }
 
-  gcry_md_write (mdh, (const unsigned char *) in, inlen);
+  err = gcry_md_setkey (mdh, shishi_key_value (key), shishi_key_length (key));
+  if (err != GPG_ERR_NO_ERROR)
+    {
+      shishi_error_printf (handle, "Libgcrypt md setkey failed");
+      shishi_error_set (handle, gpg_strerror (err));
+      return SHISHI_CRYPTO_INTERNAL_ERROR;
+    }
+
+  gcry_md_write (mdh, in, inlen);
 
   hash = gcry_md_read (mdh, halg);
   if (hash == NULL)
-    return SHISHI_GCRYPT_ERROR;
+    {
+      shishi_error_printf (handle, "Libgcrypt failed to compute hash");
+      return SHISHI_CRYPTO_INTERNAL_ERROR;
+    }
 
-  memcpy (outhash, hash, outhashlen < hlen ? outhashlen : hlen);
+  *outhashlen = hlen;
+  *outhash = xmalloc (*outhashlen);
+  memcpy (*outhash, hash, *outhashlen);
 
   gcry_md_close (mdh);
-
+#else
+  struct hmac_sha1_ctx ctx;
+  hmac_sha1_set_key (&ctx, shishi_key_length (key), shishi_key_value (key));
+  hmac_sha1_update (&ctx, inlen, in);
+  *outhashlen = SHA1_DIGEST_SIZE;
+  *outhash = xmalloc (*outhashlen);
+  hmac_sha1_digest (&ctx, *outhashlen, *outhash);
+#endif
   return SHISHI_OK;
 }
 
@@ -251,17 +278,22 @@ simplified_hmac_verify (Shishi * handle,
 			const char *in, size_t inlen,
 			const char *hmac, size_t hmaclen)
 {
-  char hash[MAX_HASH_LEN];
+  char *hash;
+  size_t hlen;
+  int same;
   int res;
 
-  res = simplified_hmac (handle, key, in, inlen, hash, hmaclen);
-  if (res != SHISHI_OK)
+  res = simplified_hmac (handle, key, in, inlen, &hash, &hlen);
+  if (res != SHISHI_OK || hash == NULL)
     return res;
 
-  if (memcmp (hash, hmac, hmaclen) != 0)
+  same = (hlen == hmaclen) && memcmp (hash, hmac, hmaclen) == 0;
+
+  free (hash);
+
+  if (!same)
     {
-      if (VERBOSE (handle))
-	printf ("simplified hmac verify fail\n");
+      shishi_error_printf (handle, "HMAC verify failed");
       return SHISHI_CRYPTO_ERROR;
     }
 
@@ -279,11 +311,11 @@ Shishi_derivekeymode;
 static int
 simplified_derivekey (Shishi * handle,
 		      Shishi_key * key,
-		      int keyusage,
-		      int derivekeymode, Shishi_key * derivedkey)
+		      int keyusage, int derivekeymode, Shishi_key ** outkey)
 {
   char constant[5];
   int res = SHISHI_OK;
+  Shishi_key *derivedkey;
 
   if (VERBOSECRYPTO (handle))
     {
@@ -297,7 +329,13 @@ simplified_derivekey (Shishi * handle,
       puts ("");
     }
 
-  shishi_key_type_set (derivedkey, shishi_key_type (key));
+
+  res = shishi_key_from_value (handle, shishi_key_type (key),
+			       NULL, &derivedkey);
+  if (res != SHISHI_OK)
+    return res;
+
+  *outkey = derivedkey;
 
   if (keyusage)
     {
@@ -334,61 +372,148 @@ simplified_dencrypt (Shishi * handle,
 		     Shishi_key * key,
 		     const char *iv, size_t ivlen,
 		     const char *in, size_t inlen,
-		     char *out, size_t * outlen,
-		     int direction)
+		     char **out, size_t * outlen, int decryptp)
 {
-  int res;
-  GCRY_CIPHER_HD ch;
+#ifdef USE_GCRYPT
+  gcry_cipher_hd_t ch;
+  gpg_error_t err;
   int alg = 0;
   int mode = GCRY_CIPHER_MODE_CBC;
   int flags = 0;
 
   switch (shishi_key_type (key))
     {
-    case SHISHI_DES3_CBC_HMAC_SHA1_KD:
-      alg = GCRY_CIPHER_3DES;
-      break;
-
     case SHISHI_DES_CBC_CRC:
     case SHISHI_DES_CBC_MD4:
     case SHISHI_DES_CBC_MD5:
       alg = GCRY_CIPHER_DES;
+      *outlen = inlen;
+      break;
+
+    case SHISHI_DES3_CBC_HMAC_SHA1_KD:
+      alg = GCRY_CIPHER_3DES;
+      *outlen = inlen;
       break;
 
     case SHISHI_AES128_CTS_HMAC_SHA1_96:
     case SHISHI_AES256_CTS_HMAC_SHA1_96:
       alg = GCRY_CIPHER_AES;
-      flags = GCRY_CIPHER_CBC_CTS;
+      flags |= GCRY_CIPHER_CBC_CTS;
+      *outlen = inlen;
       break;
     }
 
-  ch = gcry_cipher_open (alg, mode, flags);
-  if (ch == NULL)
-    return SHISHI_GCRYPT_ERROR;
-
-  res =
-    gcry_cipher_setkey (ch, shishi_key_value (key), shishi_key_length (key));
-  if (res == GCRYERR_SUCCESS)
-    res = gcry_cipher_setiv (ch, iv, ivlen);
-
-  if (res == GCRYERR_SUCCESS)
-    res = direction ?
-      gcry_cipher_decrypt (ch, (unsigned char *) out, *outlen,
-			   (const unsigned char *) in, inlen) :
-      gcry_cipher_encrypt (ch, (unsigned char *) out, *outlen,
-			   (const unsigned char *) in, inlen);
-
-  if (res != GCRYERR_SUCCESS)
+  err = gcry_cipher_open (&ch, alg, mode, flags);
+  if (err != GPG_ERR_NO_ERROR)
     {
-      puts (gcry_strerror (res));
-      shishi_error_set (handle, gcry_strerror (res));
-      return SHISHI_GCRYPT_ERROR;
+      shishi_error_printf (handle, "Libgcrypt cipher open failed");
+      shishi_error_set (handle, gpg_strerror (err));
+      return SHISHI_CRYPTO_INTERNAL_ERROR;
     }
 
-  *outlen = inlen;
+  err = gcry_cipher_setkey (ch, shishi_key_value (key),
+			    shishi_key_length (key));
+  if (err != GPG_ERR_NO_ERROR)
+    {
+      shishi_error_printf (handle, "Libgcrypt setkey failed");
+      shishi_error_set (handle, gpg_strerror (err));
+      return SHISHI_CRYPTO_INTERNAL_ERROR;
+    }
+
+  err = gcry_cipher_setiv (ch, iv, ivlen);
+  if (err != GPG_ERR_NO_ERROR)
+    {
+      shishi_error_printf (handle, "Libgcrypt setiv failed");
+      shishi_error_set (handle, gpg_strerror (err));
+      return SHISHI_CRYPTO_INTERNAL_ERROR;
+    }
+
+  *out = xmalloc (*outlen);
+
+  if (decryptp)
+    err = gcry_cipher_decrypt (ch, (unsigned char *) *out, *outlen,
+			       (const unsigned char *) in, inlen);
+  else
+    err = gcry_cipher_encrypt (ch, (unsigned char *) *out, *outlen,
+			       (const unsigned char *) in, inlen);
+  if (err != GPG_ERR_NO_ERROR)
+    {
+      shishi_error_set (handle, gpg_strerror (err));
+      return SHISHI_CRYPTO_INTERNAL_ERROR;
+    }
 
   gcry_cipher_close (ch);
 
+#else
+  struct CBC_CTX(struct des_ctx, DES_BLOCK_SIZE) des;
+  struct CBC_CTX(struct des3_ctx, DES3_BLOCK_SIZE) des3;
+  struct CBC_CTS_CTX(struct aes_ctx, AES_BLOCK_SIZE) aes;
+  void *p;
+  int rc;
+
+  *outlen = inlen;
+  *out = xmalloc (*outlen);
+
+  switch (shishi_key_type (key))
+    {
+    case SHISHI_DES_CBC_CRC:
+    case SHISHI_DES_CBC_MD4:
+    case SHISHI_DES_CBC_MD5:
+      rc = des_set_key (&des.ctx, shishi_key_value (key));
+      if (!rc)
+	{
+	  shishi_error_printf (handle, "Nettle setkey failed");
+	  return SHISHI_CRYPTO_INTERNAL_ERROR;
+	}
+      if (iv)
+	CBC_SET_IV (&des, iv);
+      else
+	memset(des.iv, 0, sizeof(des.iv));
+      if (decryptp)
+	CBC_DECRYPT (&des, des_decrypt, inlen, *out, in);
+      else
+	CBC_ENCRYPT (&des, des_encrypt, inlen, *out, in);
+      break;
+
+    case SHISHI_DES3_CBC_HMAC_SHA1_KD:
+      rc = des3_set_key (&des3.ctx, shishi_key_value (key));
+      if (!rc)
+	{
+	  shishi_error_printf (handle, "Nettle setkey failed");
+	  return SHISHI_CRYPTO_INTERNAL_ERROR;
+	}
+      if (iv)
+	CBC_SET_IV (&des3, iv);
+      else
+	memset(des3.iv, 0, sizeof(des3.iv));
+      if (decryptp)
+	CBC_DECRYPT (&des3, des3_decrypt, inlen, *out, in);
+      else
+	CBC_ENCRYPT (&des3, des3_encrypt, inlen, *out, in);
+      break;
+
+    case SHISHI_AES128_CTS_HMAC_SHA1_96:
+    case SHISHI_AES256_CTS_HMAC_SHA1_96:
+      if (iv)
+	CBC_CTS_SET_IV (&aes, iv);
+      else
+	memset(aes.iv, 0, sizeof(aes.iv));
+      if (decryptp)
+	{
+	  aes_set_decrypt_key (&aes.ctx, shishi_key_length (key),
+			       shishi_key_value (key));
+	  CBC_CTS_DECRYPT (&aes, aes_decrypt, inlen, *out, in);
+	}
+      else
+	{
+	  aes_set_encrypt_key (&aes.ctx, shishi_key_length (key),
+			       shishi_key_value (key));
+	  CBC_CTS_ENCRYPT (&aes, aes_encrypt, inlen, *out, in);
+	}
+      break;
+    }
+
+#endif
   return SHISHI_OK;
 }
 
@@ -397,8 +522,7 @@ simplified_encrypt (Shishi * handle,
 		    Shishi_key * key,
 		    int keyusage,
 		    const char *iv, size_t ivlen,
-		    const char *in, size_t inlen,
-		    char *out, size_t * outlen)
+		    const char *in, size_t inlen, char **out, size_t * outlen)
 {
   int res;
   int padzerolen = 0;
@@ -412,56 +536,59 @@ simplified_encrypt (Shishi * handle,
 
   if (keyusage != 0)
     {
-      char *buffer;
-      int buflen;
+      char *pt = NULL, *ct = NULL, *hmac = NULL;
       int blen = shishi_cipher_blocksize (shishi_key_type (key));
-      int halg = GCRY_MD_SHA1;
-      int hlen = gcry_md_get_algo_dlen (halg);
-      size_t len;
-      Shishi_key *derivedkey;
+      size_t ctlen, ptlen, hmaclen;
+      Shishi_key *privacykey = NULL, *integritykey = NULL;
 
-      res = shishi_key_from_value (handle, shishi_key_type (key),
-				   NULL, &derivedkey);
+      ptlen = inlen + blen + padzerolen;
+      pt = xmalloc (ptlen);
+
+      res = shishi_randomize (handle, pt, blen);
       if (res != SHISHI_OK)
-	return res;
+	goto done;
 
-      buflen = inlen + blen + padzerolen;
-      buffer = malloc (buflen);
-      if (!buffer)
-	return SHISHI_MALLOC_ERROR;
-
-      res = shishi_randomize (handle, buffer, blen);
-      if (res != SHISHI_OK)
-	return res;
-
-      memcpy (buffer + blen, in, inlen);
-      memset (buffer + blen + inlen, 0, padzerolen);
+      memcpy (pt + blen, in, inlen);
+      memset (pt + blen + inlen, 0, padzerolen);
 
       res = simplified_derivekey (handle, key, keyusage,
-				  SHISHI_DERIVEKEYMODE_PRIVACY, derivedkey);
+				  SHISHI_DERIVEKEYMODE_PRIVACY, &privacykey);
       if (res != SHISHI_OK)
-	return res;
+	goto done;
 
-      len = *outlen;
-      res = simplified_dencrypt (handle, derivedkey, iv, ivlen,
-				 buffer, buflen, out, &len, 0);
+      res = simplified_dencrypt (handle, privacykey, iv, ivlen,
+				 pt, ptlen, &ct, &ctlen, 0);
       if (res != SHISHI_OK)
-	return res;
+	goto done;
+
 
       res = simplified_derivekey (handle, key, keyusage,
-				  SHISHI_DERIVEKEYMODE_INTEGRITY, derivedkey);
+				  SHISHI_DERIVEKEYMODE_INTEGRITY,
+				  &integritykey);
       if (res != SHISHI_OK)
-	return res;
+	goto done;
 
-      res = simplified_hmac (handle, derivedkey, buffer, buflen,
-			     out + len, hlen);
+      res =
+	simplified_hmac (handle, integritykey, pt, ptlen, &hmac, &hmaclen);
       if (res != SHISHI_OK)
-	return res;
+	goto done;
 
-      free(buffer);
-      shishi_key_done (&derivedkey);
+      *outlen = ctlen + hmaclen;
+      *out = xmalloc (*outlen);
+      memcpy (*out, ct, ctlen);
+      memcpy (*out + ctlen, hmac, hmaclen);
 
-      *outlen = buflen + hlen;
+    done:
+      if (&privacykey)
+	shishi_key_done (&privacykey);
+      if (&integritykey)
+	shishi_key_done (&integritykey);
+      if (hmac)
+	free (hmac);
+      if (ct)
+	free (ct);
+      if (pt)
+	free (pt);
     }
   else
     {
@@ -477,51 +604,48 @@ simplified_decrypt (Shishi * handle,
 		    Shishi_key * key,
 		    int keyusage,
 		    const char *iv, size_t ivlen,
-		    const char *in, size_t inlen,
-		    char *out, size_t * outlen)
+		    const char *in, size_t inlen, char **out, size_t * outlen)
 {
   int res;
 
   if (keyusage)
     {
-      Shishi_key *derivedkey;
+      Shishi_key *privacykey = NULL, *integritykey = NULL;
       int blen = shishi_cipher_blocksize (shishi_key_type (key));
-      int halg = GCRY_MD_SHA1;
-      size_t hlen = gcry_md_get_algo_dlen (halg);
+      size_t hlen = 20; /* XXX only works for SHA-1 */
       size_t len;
 
-      res = shishi_key_from_value (handle, shishi_key_type (key),
-				   NULL, &derivedkey);
+      res = simplified_derivekey (handle, key, keyusage,
+				  SHISHI_DERIVEKEYMODE_PRIVACY, &privacykey);
       if (res != SHISHI_OK)
-	return res;
+	goto done;
+
+      res = simplified_dencrypt (handle, privacykey, iv, ivlen,
+				 in, inlen - hlen, out, outlen, 1);
+      if (res != SHISHI_OK)
+	goto done;
 
       res = simplified_derivekey (handle, key, keyusage,
-				  SHISHI_DERIVEKEYMODE_PRIVACY, derivedkey);
+				  SHISHI_DERIVEKEYMODE_INTEGRITY,
+				  &integritykey);
       if (res != SHISHI_OK)
-	return res;
+	goto done;
 
-      len = *outlen;
-      *outlen = 0;
-      res = simplified_dencrypt (handle, derivedkey, iv, ivlen,
-				 in, inlen - hlen, out, &len, 1);
-      if (res != SHISHI_OK)
-	return res;
-
-      res = simplified_derivekey (handle, key, keyusage,
-				  SHISHI_DERIVEKEYMODE_INTEGRITY, derivedkey);
-      if (res != SHISHI_OK)
-	return res;
-
-      res = simplified_hmac_verify (handle, derivedkey, out, len,
+      res = simplified_hmac_verify (handle, integritykey, *out, *outlen,
 				    in + inlen - hlen, hlen);
 
       if (res != SHISHI_OK)
-	return res;
+	goto done;
 
-      shishi_key_done (&derivedkey);
+      memmove (*out, *out + blen, *outlen - blen);
+      *outlen = *outlen - blen;
+      *out = xrealloc (*out, *outlen);
 
-      memmove (out, out + blen, len - blen);
-      *outlen = len - blen;
+    done:
+      if (privacykey)
+	shishi_key_done (&privacykey);
+      if (integritykey)
+	shishi_key_done (&integritykey);
     }
   else
     {
@@ -536,32 +660,46 @@ static int
 simplified_checksum (Shishi * handle,
 		     Shishi_key * key,
 		     int keyusage,
-		     char *in, int inlen, char *out, int *outlen)
+		     int cksumtype,
+		     char *in, size_t inlen, char **out, size_t * outlen)
 {
-  Shishi_key *derivedkey;
-  int halg = GCRY_MD_SHA1;	/* XXX hide this in crypto-lowlevel.c */
-  int hlen = gcry_md_get_algo_dlen (halg);
+  Shishi_key *checksumkey;
+  size_t hlen = 20; /* XXX only works for SHA-1 */
+  int cksumlen = shishi_checksum_cksumlen (cksumtype);
   int res;
 
-  res = shishi_key_from_value (handle, shishi_key_type (key),
-			       NULL, &derivedkey);
-  if (res != SHISHI_OK)
-    return res;
-
   res = simplified_derivekey (handle, key, keyusage,
-			      SHISHI_DERIVEKEYMODE_CHECKSUM, derivedkey);
+			      SHISHI_DERIVEKEYMODE_CHECKSUM, &checksumkey);
   if (res != SHISHI_OK)
     return res;
 
-  res = simplified_hmac (handle, derivedkey, in, inlen, out, hlen);
-  if (res != SHISHI_OK)
-    {
-      shishi_error_set (handle, "verify failed");
-      return res;
-    }
-  *outlen = hlen;
+  res = simplified_hmac (handle, checksumkey, in, inlen, out, outlen);
 
-  shishi_key_done (&derivedkey);
+  shishi_key_done (&checksumkey);
+
+  if (res != SHISHI_OK)
+    return res;
+
+  *outlen = cksumlen;
+
+  return SHISHI_OK;
+}
+
+int
+_shishi_cipher_init (void)
+{
+#ifdef USE_GCRYPT
+  if (gcry_control (GCRYCTL_ANY_INITIALIZATION_P) == 0)
+    {
+      if (gcry_check_version (GCRYPT_VERSION) == NULL)
+	return SHISHI_CRYPTO_INTERNAL_ERROR;
+      if (gcry_control (GCRYCTL_DISABLE_SECMEM, NULL, 0) != GPG_ERR_NO_ERROR)
+	return SHISHI_CRYPTO_INTERNAL_ERROR;
+      if (gcry_control (GCRYCTL_INITIALIZATION_FINISHED,
+			NULL, 0) != GPG_ERR_NO_ERROR)
+	return SHISHI_CRYPTO_INTERNAL_ERROR;
+    }
+#endif
 
   return SHISHI_OK;
 }
@@ -584,14 +722,21 @@ typedef int (*Shishi_encrypt_function) (Shishi * handle,
 					int keyusage,
 					const char *iv, size_t ivlen,
 					const char *in, size_t inlen,
-					char *out, size_t * outlen);
+					char **out, size_t * outlen);
 
 typedef int (*Shishi_decrypt_function) (Shishi * handle,
 					Shishi_key * key,
 					int keyusage,
 					const char *iv, size_t ivlen,
 					const char *in, size_t inlen,
-					char *out, size_t * outlen);
+					char **out, size_t * outlen);
+
+typedef int (*Shishi_checksum_function) (Shishi * handle,
+					 Shishi_key * key,
+					 int keyusage,
+					 int cksumtype,
+					 char *in, size_t inlen,
+					 char **out, size_t * outlen);
 
 #include "crypto-null.c"
 #include "crypto-des.c"
@@ -600,7 +745,7 @@ typedef int (*Shishi_decrypt_function) (Shishi * handle,
 
 struct cipherinfo
 {
-  int type;
+  int32_t type;
   char *name;
   int blocksize;
   int minpadsize;
@@ -701,8 +846,8 @@ static cipherinfo des3_cbc_sha1_kd_info = {
   SHISHI_HMAC_SHA1_DES3_KD,
   des3_random_to_key,
   des3_string_to_key,
-  des3_encrypt,
-  des3_decrypt
+  _des3_encrypt,
+  _des3_decrypt
 };
 
 static cipherinfo des3_cbc_none_info = {
@@ -762,23 +907,6 @@ static cipherinfo *ciphers[] = {
   &aes256_cts_hmac_sha1_96_info
 };
 
-int
-_shishi_cipher_init (void)
-{
-  if (gcry_control (GCRYCTL_ANY_INITIALIZATION_P) == 0)
-    {
-      if (gcry_check_version (GCRYPT_VERSION) == NULL)
-	return SHISHI_GCRYPT_ERROR;
-      if (gcry_control (GCRYCTL_DISABLE_SECMEM, NULL, 0) != GCRYERR_SUCCESS)
-	return SHISHI_GCRYPT_ERROR;
-      if (gcry_control (GCRYCTL_INITIALIZATION_FINISHED,
-			NULL, 0) != GCRYERR_SUCCESS)
-	return SHISHI_GCRYPT_ERROR;
-    }
-
-  return SHISHI_OK;
-}
-
 /**
  * shishi_cipher_supported_p:
  * @type: encryption type, see Shishi_etype.
@@ -786,7 +914,7 @@ _shishi_cipher_init (void)
  * Return value: Return 0 iff cipher is unsupported.
  **/
 int
-shishi_cipher_supported_p (int type)
+shishi_cipher_supported_p (int32_t type)
 {
   size_t i;
 
@@ -805,7 +933,7 @@ shishi_cipher_supported_p (int type)
  * e.g. "des3-cbc-sha1-kd", as defined in the standards.
  **/
 const char *
-shishi_cipher_name (int type)
+shishi_cipher_name (int32_t type)
 {
   size_t i;
   char *p;
@@ -816,7 +944,7 @@ shishi_cipher_name (int type)
 	return ciphers[i]->name;
     }
 
-  shishi_asprintf (&p, "unknown cipher %d", type);
+  asprintf (&p, "unknown cipher %d", type);
   return p;
 }
 
@@ -828,7 +956,7 @@ shishi_cipher_name (int type)
  * the standards.
  **/
 int
-shishi_cipher_blocksize (int type)
+shishi_cipher_blocksize (int32_t type)
 {
   size_t i;
 
@@ -847,7 +975,7 @@ shishi_cipher_blocksize (int type)
  * defined in the standards.
  **/
 int
-shishi_cipher_minpadsize (int type)
+shishi_cipher_minpadsize (int32_t type)
 {
   size_t i;
 
@@ -866,7 +994,7 @@ shishi_cipher_minpadsize (int type)
  * encryption type, as defined in the standards.
  **/
 int
-shishi_cipher_confoundersize (int type)
+shishi_cipher_confoundersize (int32_t type)
 {
   size_t i;
 
@@ -885,7 +1013,7 @@ shishi_cipher_confoundersize (int type)
  * defined in the standards.
  **/
 size_t
-shishi_cipher_keylen (int type)
+shishi_cipher_keylen (int32_t type)
 {
   size_t i;
 
@@ -904,7 +1032,7 @@ shishi_cipher_keylen (int type)
  * as defined in the standards.
  **/
 size_t
-shishi_cipher_randomlen (int type)
+shishi_cipher_randomlen (int32_t type)
 {
   size_t i;
 
@@ -923,7 +1051,7 @@ shishi_cipher_randomlen (int type)
  * encryption type, as defined in the standards.
  **/
 int
-shishi_cipher_defaultcksumtype (int type)
+shishi_cipher_defaultcksumtype (int32_t type)
 {
   size_t i;
 
@@ -959,7 +1087,7 @@ shishi_cipher_parse (const char *cipher)
 }
 
 static Shishi_random_to_key_function
-_shishi_cipher_random_to_key (int type)
+_shishi_cipher_random_to_key (int32_t type)
 {
   size_t i;
 
@@ -971,7 +1099,7 @@ _shishi_cipher_random_to_key (int type)
 }
 
 static Shishi_string_to_key_function
-_shishi_cipher_string_to_key (int type)
+_shishi_cipher_string_to_key (int32_t type)
 {
   size_t i;
 
@@ -983,7 +1111,7 @@ _shishi_cipher_string_to_key (int type)
 }
 
 static Shishi_encrypt_function
-_shishi_cipher_encrypt (int type)
+_shishi_cipher_encrypt (int32_t type)
 {
   size_t i;
 
@@ -995,7 +1123,7 @@ _shishi_cipher_encrypt (int type)
 }
 
 static Shishi_decrypt_function
-_shishi_cipher_decrypt (int type)
+_shishi_cipher_decrypt (int32_t type)
 {
   size_t i;
 
@@ -1006,6 +1134,162 @@ _shishi_cipher_decrypt (int type)
   return NULL;
 }
 
+struct checksuminfo
+{
+  int32_t type;
+  char *name;
+  int cksumlen;
+  Shishi_checksum_function checksum;
+};
+typedef struct checksuminfo checksuminfo;
+
+static checksuminfo md4_info = {
+  SHISHI_RSA_MD4_DES,
+  "rsa-md4-des",
+  24,
+  des_md4_checksum
+};
+
+static checksuminfo md5_info = {
+  SHISHI_RSA_MD5_DES,
+  "rsa-md5-des",
+  24,
+  des_md5_checksum
+};
+
+static checksuminfo md5_gss_info = {
+  SHISHI_RSA_MD5_DES_GSS,
+  "rsa-md5-des-gss",
+  8,
+  gss_des_checksum
+};
+
+static checksuminfo hmac_sha1_des3_kd_info = {
+  SHISHI_HMAC_SHA1_DES3_KD,
+  "hmac-sha1-des3-kd",
+  20,
+  des3_checksum
+};
+
+static checksuminfo hmac_sha1_96_aes128_info = {
+  SHISHI_HMAC_SHA1_96_AES128,
+  "hmac-sha1-96-aes128",
+  96 / 8,
+  aes128_checksum
+};
+
+static checksuminfo hmac_sha1_96_aes256_info = {
+  SHISHI_HMAC_SHA1_96_AES256,
+  "hmac-sha1-96-aes256",
+  96 / 8,
+  aes256_checksum
+};
+
+static checksuminfo *checksums[] = {
+  &md4_info,
+  &md5_info,
+  &md5_gss_info,
+  &hmac_sha1_des3_kd_info,
+  &hmac_sha1_96_aes128_info,
+  &hmac_sha1_96_aes256_info
+};
+
+/**
+ * shishi_checksum_supported_p:
+ * @type: encryption type, see Shishi_etype.
+ *
+ * Return value: Return 0 iff checksum is unsupported.
+ **/
+int
+shishi_checksum_supported_p (int32_t type)
+{
+  size_t i;
+
+  for (i = 0; i < sizeof (checksums) / sizeof (checksums[0]); i++)
+    if (type == checksums[i]->type)
+      return 1;
+
+  return 0;
+}
+
+/**
+ * shishi_checksum_name:
+ * @type: encryption type, see Shishi_etype.
+ *
+ * Return value: Return name of checksum type,
+ * e.g. "hmac-sha1-96-aes256", as defined in the standards.
+ **/
+const char *
+shishi_checksum_name (int32_t type)
+{
+  size_t i;
+  char *p;
+
+  for (i = 0; i < sizeof (checksums) / sizeof (checksums[0]); i++)
+    {
+      if (type == checksums[i]->type)
+	return checksums[i]->name;
+    }
+
+  asprintf (&p, "unknown checksum %d", type);
+  return p;
+}
+
+/**
+ * shishi_checksum_cksumlen:
+ * @type: encryption type, see Shishi_etype.
+ *
+ * Return value: Return length of checksum used for the encryption type,
+ * as defined in the standards.
+ **/
+size_t
+shishi_checksum_cksumlen (int32_t type)
+{
+  size_t i;
+
+  for (i = 0; i < sizeof (checksums) / sizeof (checksums[0]); i++)
+    if (type == checksums[i]->type)
+      return checksums[i]->cksumlen;
+
+  return -1;
+}
+
+/**
+ * shishi_checksum_parse:
+ * @checksum: name of checksum type, e.g. "hmac-sha1-96-aes256".
+ *
+ * Return value: Return checksum type corresponding to a string.
+ **/
+int
+shishi_checksum_parse (const char *checksum)
+{
+  size_t i;
+  char *endptr;
+
+  i = strtol (checksum, &endptr, 0);
+
+  if (endptr != checksum)
+    return i;
+
+  for (i = 0; i < sizeof (checksums) / sizeof (checksums[0]); i++)
+    if (strcasecmp (checksum, checksums[i]->name) == 0)
+      return checksums[i]->type;
+
+  return -1;
+}
+
+static Shishi_checksum_function
+_shishi_checksum (int32_t type)
+{
+  size_t i;
+
+  for (i = 0; i < sizeof (checksums) / sizeof (checksums[0]); i++)
+    if (type == checksums[i]->type)
+      return checksums[i]->checksum;
+
+  return NULL;
+}
+
 /**
  * shishi_string_to_key:
  * @handle: shishi handle as allocated by shishi_init().
@@ -1027,11 +1311,10 @@ _shishi_cipher_decrypt (int type)
  **/
 int
 shishi_string_to_key (Shishi * handle,
-		      int keytype,
-		      const char *password,
-		      int passwordlen,
-		      const char *salt,
-		      int saltlen, const char *parameter, Shishi_key * outkey)
+		      int32_t keytype,
+		      const char *password, size_t passwordlen,
+		      const char *salt, size_t saltlen,
+		      const char *parameter, Shishi_key * outkey)
 {
   Shishi_string_to_key_function string2key;
   int res;
@@ -1057,7 +1340,7 @@ shishi_string_to_key (Shishi * handle,
     {
       shishi_error_printf (handle, "Unsupported keytype %d",
 			   shishi_key_type (outkey));
-      return !SHISHI_OK;
+      return SHISHI_CRYPTO_ERROR;
     }
 
   res = (*string2key) (handle, password, passwordlen,
@@ -1090,8 +1373,8 @@ shishi_string_to_key (Shishi * handle,
  **/
 int
 shishi_random_to_key (Shishi * handle,
-		      int keytype,
-		      char *random, int randomlen, Shishi_key * outkey)
+		      int32_t keytype,
+		      char *random, size_t randomlen, Shishi_key * outkey)
 {
   Shishi_random_to_key_function random2key;
   int res;
@@ -1113,7 +1396,7 @@ shishi_random_to_key (Shishi * handle,
     {
       shishi_error_printf (handle, "Unsupported random_to_key() ekeytype %d",
 			   keytype);
-      return !SHISHI_OK;
+      return SHISHI_CRYPTO_ERROR;
     }
 
   res = (*random2key) (handle, random, randomlen, outkey);
@@ -1154,8 +1437,10 @@ int
 shishi_checksum (Shishi * handle,
 		 Shishi_key * key,
 		 int keyusage,
-		 int cksumtype, char *in, int inlen, char *out, int *outlen)
+		 int cksumtype,
+		 char *in, size_t inlen, char **out, size_t * outlen)
 {
+  Shishi_checksum_function checksum;
   int res;
 
   if (VERBOSECRYPTO (handle))
@@ -1174,150 +1459,20 @@ shishi_checksum (Shishi * handle,
   if (cksumtype == 0)
     cksumtype = shishi_cipher_defaultcksumtype (shishi_key_type (key));
 
-  /* XXX create a dispatcher instead of hardcoding this */
-
-  switch (cksumtype)
+  checksum = _shishi_checksum (cksumtype);
+  if (checksum == NULL)
     {
-    case SHISHI_RSA_MD4_DES:
-      {
-	char buffer[BUFSIZ];
-	int buflen;
-	char *keyp;
-	int i;
-
-	buflen = sizeof (buffer);
-	res = checksum_md4 (handle, buffer, &buflen, in, inlen);
-	if (res != SHISHI_OK)
-	  {
-	    shishi_error_set (handle, "checksum failed");
-	    return res;
-	  }
-
-	keyp = shishi_key_value (key);
-
-	for (i = 0; i < 8; i++)
-	  keyp[i] ^= 0xF0;
-
-	res = simplified_dencrypt (handle, key, NULL, 0, buffer, buflen,
-				   out, outlen, 0);
-
-	for (i = 0; i < 8; i++)
-	  keyp[i] ^= 0xF0;
-
-	if (res != SHISHI_OK)
-	  {
-	    shishi_error_set (handle, "encrypt failed");
-	    return res;
-	  }
-      }
-      break;
-
-    case SHISHI_RSA_MD5_DES:
-      {
-	char buffer[BUFSIZ];
-	int buflen;
-	char *keyp;
-	int i;
-
-	buflen = sizeof (buffer);
-	res = checksum_md5 (handle, buffer, &buflen, in, inlen);
-	if (res != SHISHI_OK)
-	  {
-	    shishi_error_set (handle, "checksum failed");
-	    return res;
-	  }
-
-	keyp = shishi_key_value (key);
-
-	for (i = 0; i < 8; i++)
-	  keyp[i] ^= 0xF0;
-
-	res = simplified_dencrypt (handle, key, NULL, 0, buffer, buflen,
-				   out, outlen, 0);
-
-	for (i = 0; i < 8; i++)
-	  keyp[i] ^= 0xF0;
-
-	if (res != SHISHI_OK)
-	  {
-	    shishi_error_set (handle, "encrypt failed");
-	    return res;
-	  }
-      }
-      break;
-
-    case SHISHI_HMAC_SHA1_DES3_KD:
-      res =
-	simplified_checksum (handle, key, keyusage, in, inlen, out, outlen);
-      break;
-
-    case SHISHI_HMAC_SHA1_96_AES128:
-      res =
-	simplified_checksum (handle, key, keyusage, in, inlen, out, outlen);
-      *outlen = 96 / 8;
-      break;
-
-    case SHISHI_HMAC_SHA1_96_AES256:
-      res =
-	simplified_checksum (handle, key, keyusage, in, inlen, out, outlen);
-      *outlen = 96 / 8;
-      break;
-
-    case 42:
-      {
-	char buffer[BUFSIZ];
-	int buflen;
-	char *keyp;
-	char *p;
-	int i;
-	GCRY_MD_HD hd;
-	GCRY_CIPHER_HD ch;
-
-	hd = gcry_md_open (GCRY_MD_MD5, 0);
-	if (!hd)
-	  return SHISHI_GCRYPT_ERROR;
-
-	gcry_md_write (hd, in, inlen);
-	p = gcry_md_read (hd, GCRY_MD_MD5);
-
-	keyp = shishi_key_value (key);
-
-	ch = gcry_cipher_open (GCRY_CIPHER_DES,
-			       GCRY_CIPHER_MODE_CBC,
-			       GCRY_CIPHER_CBC_MAC);
-	if (ch == NULL)
-	  return SHISHI_GCRYPT_ERROR;
-
-	res = gcry_cipher_setkey (ch, keyp, 8);
-	if (res != GCRYERR_SUCCESS)
-	  return SHISHI_GCRYPT_ERROR;
-
-	res = gcry_cipher_setiv (ch, NULL, 8);
-	if (res != 0)
-	  return SHISHI_GCRYPT_ERROR;
-
-	res = gcry_cipher_encrypt (ch, out, 8, p, 16);
-	if (res != 0)
-	  return SHISHI_GCRYPT_ERROR;
-
-	outlen = 8;
-
-	gcry_cipher_close (ch);
-	gcry_md_close (hd);
-      }
-      break;
-
-    default:
-      res = !SHISHI_OK;
-      printf ("unimplemented checksum type!\n");
-      break;
+      shishi_error_printf (handle, "Unsupported checksum type %d", cksumtype);
+      return SHISHI_CRYPTO_ERROR;
     }
 
+  res = (*checksum) (handle, key, keyusage, cksumtype, in, inlen, out, outlen);
+
   if (VERBOSECRYPTO (handle))
     {
       printf ("\t ;; checksum out:\n");
-      escapeprint (out, *outlen);
-      hexprint (out, *outlen);
+      escapeprint (*out, *outlen);
+      hexprint (*out, *outlen);
       puts ("");
     }
 
@@ -1350,10 +1505,9 @@ int
 shishi_encrypt_iv_etype (Shishi * handle,
 			 Shishi_key * key,
 			 int keyusage,
-			 int etype,
-			 char *iv, int ivlen,
-			 char *in, int inlen,
-			 char *out, int *outlen)
+			 int32_t etype,
+			 char *iv, size_t ivlen,
+			 char *in, size_t inlen, char **out, size_t * outlen)
 {
   Shishi_encrypt_function encrypt;
   int res;
@@ -1376,7 +1530,7 @@ shishi_encrypt_iv_etype (Shishi * handle,
     {
       shishi_error_printf (handle, "Unsupported keytype %d",
 			   shishi_key_type (key));
-      return !SHISHI_OK;
+      return SHISHI_CRYPTO_ERROR;
     }
 
   res = (*encrypt) (handle, key, keyusage, iv, ivlen, in, inlen, out, outlen);
@@ -1384,8 +1538,8 @@ shishi_encrypt_iv_etype (Shishi * handle,
   if (VERBOSECRYPTO (handle))
     {
       printf ("\t ;; encrypt out:\n");
-      escapeprint (out, *outlen);
-      hexprint (out, *outlen);
+      escapeprint (*out, *outlen);
+      hexprint (*out, *outlen);
       puts ("");
     }
 
@@ -1415,12 +1569,12 @@ int
 shishi_encrypt_iv (Shishi * handle,
 		   Shishi_key * key,
 		   int keyusage,
-		   char *iv, int ivlen,
-		   char *in, int inlen,
-		   char *out, int *outlen)
+		   char *iv, size_t ivlen,
+		   char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return shishi_encrypt_iv_etype (handle, key, keyusage, shishi_key_type (key),
-				  NULL, 0, in, inlen, out, outlen);
+  return shishi_encrypt_iv_etype (handle, key, keyusage,
+				  shishi_key_type (key), NULL, 0, in, inlen,
+				  out, outlen);
 }
 
 /**
@@ -1445,7 +1599,8 @@ shishi_encrypt_iv (Shishi * handle,
 int
 shishi_encrypt (Shishi * handle,
 		Shishi_key * key,
-		int keyusage, char *in, int inlen, char *out, int *outlen)
+		int keyusage,
+		char *in, size_t inlen, char **out, size_t * outlen)
 {
   return shishi_encrypt_iv (handle, key, keyusage, NULL, 0,
 			    in, inlen, out, outlen);
@@ -1477,10 +1632,9 @@ int
 shishi_decrypt_iv_etype (Shishi * handle,
 			 Shishi_key * key,
 			 int keyusage,
-			 int etype,
-			 char *iv, int ivlen,
-			 char *in, int inlen,
-			 char *out, int *outlen)
+			 int32_t etype,
+			 char *iv, size_t ivlen,
+			 char *in, size_t inlen, char **out, size_t * outlen)
 {
   Shishi_decrypt_function decrypt;
   int res;
@@ -1503,7 +1657,7 @@ shishi_decrypt_iv_etype (Shishi * handle,
     {
       shishi_error_printf (handle, "Unsupported keytype %d",
 			   shishi_key_type (key));
-      return !SHISHI_OK;
+      return SHISHI_CRYPTO_ERROR;
     }
 
   res = (*decrypt) (handle, key, keyusage, iv, ivlen, in, inlen, out, outlen);
@@ -1511,8 +1665,8 @@ shishi_decrypt_iv_etype (Shishi * handle,
   if (VERBOSECRYPTO (handle))
     {
       printf ("\t ;; decrypt out:\n");
-      escapeprint (out, *outlen);
-      hexprint (out, *outlen);
+      escapeprint (*out, *outlen);
+      hexprint (*out, *outlen);
       puts ("");
     }
 
@@ -1544,15 +1698,12 @@ int
 shishi_decrypt_iv (Shishi * handle,
 		   Shishi_key * key,
 		   int keyusage,
-		   char *iv, int ivlen,
-		   char *in, int inlen,
-		   char *out, int *outlen)
+		   char *iv, size_t ivlen,
+		   char *in, size_t inlen, char **out, size_t * outlen)
 {
   return shishi_decrypt_iv_etype (handle, key, keyusage,
 				  shishi_key_type (key),
-				  iv, ivlen,
-				  in, inlen,
-				  out, outlen);
+				  iv, ivlen, in, inlen, out, outlen);
 }
 
 /**
@@ -1577,7 +1728,8 @@ shishi_decrypt_iv (Shishi * handle,
 int
 shishi_decrypt (Shishi * handle,
 		Shishi_key * key,
-		int keyusage, char *in, int inlen, char *out, int *outlen)
+		int keyusage,
+		char *in, size_t inlen, char **out, size_t * outlen)
 {
   return shishi_decrypt_iv (handle, key, keyusage, NULL, 0,
 			    in, inlen, out, outlen);
@@ -1595,20 +1747,29 @@ shishi_decrypt (Shishi * handle,
  * Return value: Returns %SHISHI_OK iff successful.
  **/
 int
-shishi_randomize (Shishi * handle, char *data, int datalen)
+shishi_randomize (Shishi * handle, char *data, size_t datalen)
 {
-  char tmp[BUFSIZ];
-
-  memcpy (data, tmp, datalen < BUFSIZ ? datalen : BUFSIZ);
-
+#ifdef USE_GCRYPT
   gcry_randomize (data, datalen, GCRY_STRONG_RANDOM);
+#else
+  int fd;
+  char *device;
 
-  if (datalen > 0 &&
-      memcmp (data, tmp, datalen < BUFSIZ ? datalen : BUFSIZ) == 0)
+  device = "/dev/urandom";
+
+  fd = open (device, O_RDONLY);
+  if (fd < 0)
     {
-      shishi_error_set (handle, "No random data collected.");
-      return SHISHI_GCRYPT_ERROR;
+      shishi_error_printf(handle, "Could not open random device: %s\n",
+			  strerror (errno));
+      return SHISHI_FOPEN_ERROR;
     }
+  else
+    {
+      read (fd, data, datalen);
+      close (fd);
+    }
+#endif
 
   return SHISHI_OK;
 }
@@ -1633,7 +1794,8 @@ shishi_randomize (Shishi * handle, char *data, int datalen)
  * Return value: Returns %SHISHI_OK iff successful.
  **/
 int
-shishi_n_fold (Shishi * handle, char *in, int inlen, char *out, int outlen)
+shishi_n_fold (Shishi * handle,
+	       char *in, size_t inlen, char *out, size_t outlen)
 {
   int m = inlen;
   int n = outlen;
@@ -1776,14 +1938,14 @@ shishi_n_fold (Shishi * handle, char *in, int inlen, char *out, int outlen)
 int
 shishi_dr (Shishi * handle,
 	   Shishi_key * key,
-	   char *constant,
-	   int constantlen, char *derivedrandom, int derivedrandomlen)
+	   char *constant, size_t constantlen,
+	   char *derivedrandom, size_t derivedrandomlen)
 {
-  char cipher[MAX_DR_CONSTANT];
+  char *cipher;
   char plaintext[MAX_DR_CONSTANT];
   char nfoldconstant[MAX_DR_CONSTANT];
   int blocksize = shishi_cipher_blocksize (shishi_key_type (key));
-  int totlen, cipherlen;
+  size_t totlen, cipherlen;
   int res;
 
   if (VERBOSECRYPTO (handle))
@@ -1805,7 +1967,7 @@ shishi_dr (Shishi * handle,
     }
 
   if (constantlen > MAX_DR_CONSTANT)
-    return !SHISHI_OK;
+    return SHISHI_TOO_SMALL_BUFFER;
 
   if (constantlen == blocksize)
     {
@@ -1834,13 +1996,15 @@ shishi_dr (Shishi * handle,
   totlen = 0;
   do
     {
-      cipherlen = sizeof (cipher);
       res = shishi_encrypt (handle, key, 0, plaintext, blocksize,
-			    cipher, &cipherlen);
+			    &cipher, &cipherlen);
       if (res != SHISHI_OK)
 	return res;
+      if (cipherlen != blocksize)
+	return SHISHI_CRYPTO_ERROR;
       memcpy (derivedrandom + totlen, cipher, cipherlen);
       memcpy (plaintext, cipher, cipherlen);
+      free (cipher);
       totlen += cipherlen;
     }
   while (totlen < derivedrandomlen);
