@@ -66,8 +66,8 @@ shishi_strerror (int err)
       p = _("Could not close file.");
       break;
 
-    case SHISHI_GCRYPT_ERROR:
-      p = _("Internal libgcrypt error.");
+    case SHISHI_CRYPTO_INTERNAL_ERROR:
+      p = _("Internal error in low-level crypto routines.");
       break;
 
     case SHISHI_NONCE_MISMATCH:
@@ -147,7 +147,7 @@ shishi_strerror (int err)
       break;
 
     default:
-      shishi_asprintf (&p, _("Unknown shishi error (%d)"), err);
+      asprintf (&p, _("Unknown shishi error (%d)"), err);
       break;
     }
 
@@ -165,7 +165,12 @@ void
 shishi_error_set (Shishi * handle, const char *error)
 {
   if (error)
-    strncpy (handle->error, error, sizeof (handle->error));
+    {
+      strncpy (handle->error, error, sizeof (handle->error));
+
+      if (VERBOSE (handle))
+	puts (handle->error);
+    }
   else
     shishi_error_clear (handle);
 }
@@ -178,4 +183,7 @@ shishi_error_printf (Shishi * handle, char *format, ...)
   va_start (ap, format);
 
   vsnprintf (handle->error, sizeof (handle->error), format, ap);
+
+  if (VERBOSE (handle))
+    puts (handle->error);
 }
