@@ -65,6 +65,14 @@
 #include <sys/ioctl.h>
 #endif
 
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -112,17 +120,23 @@
 #include <arpa/inet.h>
 #endif
 
-#ifdef ENABLE_NLS
-extern char *_shishi_gettext (const char *str);
-#define _(String) _shishi_gettext (String)
-#define gettext_noop(String) String
-#define N_(String) gettext_noop (String)
+#ifdef HAVE_ARPA_NAMESER_H
+#include <arpa/nameser.h>
+#endif
+#ifdef HAVE_RESOLV_H
+#include <resolv.h>
 #endif
 
 #include "setenv.h"
+#include "vasprintf.h"
 #include "gettext.h"
+#include "xalloc.h"
 #include "shishi.h"
 
+#define _(String) dgettext (PACKAGE, String)
+#define gettext_noop(String) String
+#define N_(String) gettext_noop (String)
+
 #define GENERALIZEDTIME_TIME_LEN 15
 #define MAX_KEY_LEN 32
 #define MAX_RANDOM_LEN 32
@@ -169,13 +183,14 @@ struct Shishi
   char *default_principal;
   int kdctimeout;
   int kdcretries;
-  int *clientkdcetypes;
+  int32_t *clientkdcetypes;
   int nclientkdcetypes;
   struct Shishi_realminfo *realminfos;
   int nrealminfos;
   char *kdc;
   char error[1024];
   char gztime_buf[40];
+  char *userdirectory;
   char *usercfgfile;
   char *tktsdefaultfile;
   char *hostkeysdefaultfile;
@@ -185,7 +200,7 @@ struct Shishi
 
 #define BASE_DIR "/.shishi"
 #define TICKET_FILE BASE_DIR "/tickets"
-#define USERCFG_FILE BASE_DIR "/config"
+#define USERCFG_FILE BASE_DIR "/shishi.conf"
 
 int
 _shishi_print_armored_data (Shishi * handle,
@@ -227,4 +242,33 @@ shishi_asn1_integer2_field (Shishi * handle,
 #include <dmalloc.h>
 #endif
 
+/* older systems might not have these */
+#ifndef T_SRV
+# define T_SRV (33)
+#endif
+
+typedef struct dnshost_st
+{
+  struct dnshost_st   *next;
+
+  unsigned int        type;
+  unsigned int        class;
+  unsigned int        ttl;
+
+  void                *rr;
+} *dnshost_t;
+
+typedef struct dns_srv_st
+{
+  unsigned int        priority;
+  unsigned int        weight;
+  unsigned int        port;
+  unsigned int        rweight;
+
+  char                name[256];
+} *dns_srv_t;
+
+dnshost_t _shishi_resolv (const char *zone, unsigned int type);
+void _shishi_resolv_free (dnshost_t dns);
+
 #endif /* _INTERNAL_H */
