@@ -41,11 +41,11 @@
 int
 shishi_as_derive_salt (Shishi * handle,
 		       Shishi_asn1 asreq,
-		       Shishi_asn1 asrep, char *salt, int *saltlen)
+		       Shishi_asn1 asrep, char *salt, size_t * saltlen)
 {
   int len = *saltlen;
   int tmplen;
-  char format[BUFSIZ];
+  char *format;
   int res;
   int i, n;
 
@@ -59,15 +59,17 @@ shishi_as_derive_salt (Shishi * handle,
     {
       int patype;
 
-      sprintf (format, "padata.?%d.padata-type", i);
-      res = shishi_asn1_integer_field (handle, asrep, &patype, format);
+      asprintf (&format, "padata.?%d.padata-type", i);
+      res = shishi_asn1_read_int32 (handle, asrep, format, &patype);
+      free (format);
       if (res != SHISHI_OK)
 	return res;
 
       if (patype == SHISHI_PA_PW_SALT)
 	{
-	  sprintf (format, "padata.?%d.padata-value", i);
+	  asprintf (&format, "padata.?%d.padata-value", i);
 	  res = shishi_asn1_read (handle, asrep, format, salt, saltlen);
+	  free (format);
 	  if (res != SHISHI_OK)
 	    return res;
 
@@ -76,14 +78,12 @@ shishi_as_derive_salt (Shishi * handle,
     }
 
   len = *saltlen;
-  res =
-    shishi_asn1_read (handle, asreq, "req-body.realm", salt, &len);
+  res = shishi_asn1_read (handle, asreq, "req-body.realm", salt, &len);
   if (res != SHISHI_OK)
     return res;
 
   res = shishi_asn1_number_of_elements (handle, asreq,
-					"req-body.cname.name-string",
-					&n);
+					"req-body.cname.name-string", &n);
   if (res != SHISHI_OK)
     return res;
 
@@ -93,8 +93,9 @@ shishi_as_derive_salt (Shishi * handle,
       if (tmplen < 0)
 	return SHISHI_TOO_SMALL_BUFFER;
 
-      sprintf (format, "req-body.cname.name-string.?%d", i);
+      asprintf (&format, "req-body.cname.name-string.?%d", i);
       res = shishi_asn1_read (handle, asreq, format, salt + len, &tmplen);
+      free (format);
       if (res != SHISHI_OK)
 	return res;
 
@@ -136,7 +137,7 @@ shishi_kdcreq_sendrecv (Shishi * handle, Shishi_asn1 kdcreq,
     }
   realm[realmlen] = '\0';
 
-  out_len = sizeof(buffer);
+  out_len = sizeof (buffer);
   res = shishi_kdc_sendrecv (handle, realm, der, der_len, buffer, &out_len);
   if (res != SHISHI_OK)
     {
@@ -144,7 +145,7 @@ shishi_kdcreq_sendrecv (Shishi * handle, Shishi_asn1 kdcreq,
 			   shishi_strerror_details (handle));
       return res;
     }
-  free(der);
+  free (der);
 
   if (VERBOSEASN1 (handle))
     printf ("received %d bytes\n", out_len);
@@ -202,8 +203,7 @@ shishi_kdc_copy_crealm (Shishi * handle,
   buf[0] = '\0';		/* XXX if crealm is empty, buflen == 0 which
 				   causes libtasn1 to strlen(buf)... */
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, encticketpart, "crealm",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, encticketpart, "crealm", buf, &buflen);
   if (res != SHISHI_OK)
     return res;
 
@@ -243,8 +243,7 @@ shishi_as_check_crealm (Shishi * handle, Shishi_asn1 asreq, Shishi_asn1 asrep)
       return res;
     }
 
-  res = shishi_asn1_read (handle, asrep, "crealm",
-			  reprealm, &reprealmlen);
+  res = shishi_asn1_read (handle, asrep, "crealm", reprealm, &reprealmlen);
   if (res != SHISHI_OK)
     {
       shishi_error_printf (handle, "Could not read reply realm: %s\n",
@@ -293,26 +292,22 @@ shishi_kdc_copy_cname (Shishi * handle,
   if (res != SHISHI_OK)
     return res;
 
-  res = shishi_asn1_write (handle, kdcrep, "cname.name-type",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, kdcrep, "cname.name-type", buf, buflen);
   if (res != SHISHI_OK)
     return res;
 
   res = shishi_asn1_number_of_elements (handle, encticketpart,
-					"cname.name-string",
-					&n);
+					"cname.name-string", &n);
   if (res != SHISHI_OK)
     return res;
 
-  res = shishi_asn1_write (handle, kdcrep, "cname.name-string",
-			   NULL, 0);
+  res = shishi_asn1_write (handle, kdcrep, "cname.name-string", NULL, 0);
   if (res != SHISHI_OK)
     return res;
 
   for (i = 1; i <= n; i++)
     {
-      res = shishi_asn1_write (handle, kdcrep, "cname.name-string",
-			       "NEW", 1);
+      res = shishi_asn1_write (handle, kdcrep, "cname.name-string", "NEW", 1);
       if (res != SHISHI_OK)
 	return res;
 
@@ -356,8 +351,7 @@ shishi_as_check_cname (Shishi * handle, Shishi_asn1 asreq, Shishi_asn1 asrep)
   /* We do not compare msg-type as recommended on the ietf-krb-wg list */
 
   res = shishi_asn1_number_of_elements (handle, asreq,
-					"req-body.cname.name-string",
-					&i);
+					"req-body.cname.name-string", &i);
   if (res != SHISHI_OK)
     return res;
 
@@ -416,7 +410,7 @@ shishi_kdc_copy_nonce (Shishi * handle,
 		       Shishi_asn1 kdcreq, Shishi_asn1 enckdcreppart)
 {
   int res;
-  unsigned long nonce;
+  uint32_t nonce;
 
   res = shishi_kdcreq_nonce (handle, kdcreq, &nonce);
   if (res != SHISHI_OK)
@@ -667,8 +661,7 @@ shishi_kdc_process (Shishi * handle,
    */
 
   msgtype = 0;
-  res = shishi_asn1_integer_field (handle, kdcrep,
-				   &msgtype, "msg-type");
+  res = shishi_asn1_read_integer (handle, kdcrep, "msg-type", &msgtype);
   if (res != SHISHI_OK)
     return res;
 
