@@ -294,8 +294,7 @@ shishi_kdcrep_crealm_set (Shishi * handle,
 int
 shishi_kdcrep_cname_set (Shishi * handle,
 			 Shishi_asn1 kdcrep,
-			 Shishi_name_type name_type,
-			 char *cname[])
+			 Shishi_name_type name_type, const char *cname[])
 {
   int res;
 
@@ -318,8 +317,7 @@ shishi_kdcrep_cname_set (Shishi * handle,
  **/
 int
 shishi_kdcrep_client_set (Shishi * handle,
-			  Shishi_asn1 kdcrep,
-			  const char *client)
+			  Shishi_asn1 kdcrep, const char *client)
 {
   int res;
 
@@ -360,10 +358,9 @@ shishi_kdcrep_crealmserver_set (Shishi * handle,
  **/
 int
 shishi_kdcrep_get_enc_part_etype (Shishi * handle,
-				  Shishi_asn1 kdcrep, int *etype)
+				  Shishi_asn1 kdcrep, int32_t * etype)
 {
-  return shishi_asn1_integer_field (handle, kdcrep, etype,
-				    "enc-part.etype");
+  return shishi_asn1_read_int32 (handle, kdcrep, "enc-part.etype", etype);
 }
 
 /**
@@ -393,8 +390,7 @@ shishi_kdcrep_get_ticket (Shishi * handle,
     return SHISHI_ASN1_ERROR;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, kdcrep, "ticket.tkt-vno",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, kdcrep, "ticket.tkt-vno", buf, &buflen);
   if (res != SHISHI_OK)
     goto error;
 
@@ -403,8 +399,7 @@ shishi_kdcrep_get_ticket (Shishi * handle,
     goto error;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, kdcrep, "ticket.realm",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, kdcrep, "ticket.realm", buf, &buflen);
   if (res != SHISHI_OK)
     goto error;
 
@@ -418,14 +413,12 @@ shishi_kdcrep_get_ticket (Shishi * handle,
   if (res != SHISHI_OK)
     goto error;
 
-  res = shishi_asn1_write (handle, *ticket, "sname.name-type",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, *ticket, "sname.name-type", buf, buflen);
   if (res != SHISHI_OK)
     goto error;
 
   res = shishi_asn1_number_of_elements (handle, kdcrep,
-					"ticket.sname.name-string",
-					&n);
+					"ticket.sname.name-string", &n);
   if (res != SHISHI_OK)
     goto error;
 
@@ -454,8 +447,7 @@ shishi_kdcrep_get_ticket (Shishi * handle,
   if (res != SHISHI_OK)
     goto error;
 
-  res = shishi_asn1_write (handle, *ticket, "enc-part.etype",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, *ticket, "enc-part.etype", buf, buflen);
   if (res != SHISHI_OK)
     goto error;
 
@@ -466,11 +458,9 @@ shishi_kdcrep_get_ticket (Shishi * handle,
     goto error;
 
   if (res == SHISHI_ASN1_NO_ELEMENT)
-    res = shishi_asn1_write (handle, *ticket, "enc-part.kvno",
-			     NULL, 0);
+    res = shishi_asn1_write (handle, *ticket, "enc-part.kvno", NULL, 0);
   else
-    res = shishi_asn1_write (handle, *ticket, "enc-part.kvno",
-			     buf, buflen);
+    res = shishi_asn1_write (handle, *ticket, "enc-part.kvno", buf, buflen);
   if (res != SHISHI_OK)
     goto error;
 
@@ -480,8 +470,7 @@ shishi_kdcrep_get_ticket (Shishi * handle,
   if (res != SHISHI_OK)
     goto error;
 
-  res = shishi_asn1_write (handle, *ticket, "enc-part.cipher",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, *ticket, "enc-part.cipher", buf, buflen);
   if (res != SHISHI_OK)
     goto error;
 
@@ -517,8 +506,7 @@ shishi_kdcrep_set_ticket (Shishi * handle, Shishi_asn1 kdcrep,
   if (res != SHISHI_OK)
     return res;
 
-  res = shishi_asn1_write (handle, kdcrep, "ticket.tkt-vno",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, kdcrep, "ticket.tkt-vno", buf, buflen);
   if (res != SHISHI_OK)
     return res;
 
@@ -527,14 +515,12 @@ shishi_kdcrep_set_ticket (Shishi * handle, Shishi_asn1 kdcrep,
   if (res != SHISHI_OK)
     return res;
 
-  res = shishi_asn1_write (handle, kdcrep, "ticket.realm",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, kdcrep, "ticket.realm", buf, buflen);
   if (res != SHISHI_OK)
     return res;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, ticket, "sname.name-type",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, ticket, "sname.name-type", buf, &buflen);
   if (res != SHISHI_OK)
     return res;
 
@@ -570,8 +556,7 @@ shishi_kdcrep_set_ticket (Shishi * handle, Shishi_asn1 kdcrep,
     }
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, ticket, "enc-part.etype",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, ticket, "enc-part.etype", buf, &buflen);
   if (res != SHISHI_OK)
     return res;
 
@@ -581,11 +566,9 @@ shishi_kdcrep_set_ticket (Shishi * handle, Shishi_asn1 kdcrep,
     return res;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, ticket, "enc-part.kvno",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, ticket, "enc-part.kvno", buf, &buflen);
   if (res != SHISHI_OK)
-    res = shishi_asn1_write (handle, kdcrep, "ticket.enc-part.kvno",
-			     NULL, 0);
+    res = shishi_asn1_write (handle, kdcrep, "ticket.enc-part.kvno", NULL, 0);
   else
     res = shishi_asn1_write (handle, kdcrep, "ticket.enc-part.kvno",
 			     buf, buflen);
@@ -593,8 +576,7 @@ shishi_kdcrep_set_ticket (Shishi * handle, Shishi_asn1 kdcrep,
     return res;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, ticket, "enc-part.cipher",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, ticket, "enc-part.cipher", buf, &buflen);
   if (res != SHISHI_OK)
     return res;
 
@@ -628,34 +610,25 @@ shishi_kdcrep_set_enc_part (Shishi * handle,
 			    Shishi_asn1 kdcrep,
 			    int etype, int kvno, char *buf, int buflen)
 {
-  char *format;
   int res = SHISHI_OK;
 
-  res = shishi_asn1_write (handle, kdcrep, "enc-part.cipher",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, kdcrep, "enc-part.cipher", buf, buflen);
   if (res != SHISHI_OK)
     return res;
 
-  shishi_asprintf (&format, "%d", etype);
-  res = shishi_asn1_write (handle, kdcrep, "enc-part.etype",
-			   format, 0);
-  free (format);
+  res = shishi_asn1_write_int32 (handle, kdcrep, "enc-part.etype", etype);
   if (res != SHISHI_OK)
     return res;
 
   if (kvno == 0)
     {
-      res = shishi_asn1_write (handle, kdcrep, "enc-part.kvno",
-			       NULL, 0);
+      res = shishi_asn1_write (handle, kdcrep, "enc-part.kvno", NULL, 0);
       if (res != SHISHI_OK)
 	return res;
     }
   else
     {
-      shishi_asprintf (&format, "%d", etype);
-      res = shishi_asn1_write (handle, kdcrep, "enc-part.kvno",
-			       format, 0);
-      free (format);
+      res = shishi_asn1_write_uint32 (handle, kdcrep, "enc-part.kvno", kvno);
       if (res != SHISHI_OK)
 	return res;
     }
@@ -685,30 +658,34 @@ shishi_kdcrep_add_enc_part (Shishi * handle,
 			    int keyusage, Shishi_asn1 enckdcreppart)
 {
   int res = SHISHI_OK;
-  char buf[BUFSIZ];
-  int buflen;
-  char der[BUFSIZ];
+  char *buf;
+  size_t buflen;
+  char *der;
   size_t derlen;
 
-  res = shishi_a2d (handle, enckdcreppart, der, &derlen);
+  res = shishi_new_a2d (handle, enckdcreppart, &der, &derlen);
   if (res != SHISHI_OK)
     {
       shishi_error_printf (handle, "Could not DER encode enckdcreppart: %s\n",
 			   shishi_strerror (res));
-      return !SHISHI_OK;
+      return SHISHI_ASN1_ERROR;
     }
 
-  buflen = BUFSIZ;
-  res = shishi_encrypt (handle, key, keyusage, der, derlen, buf, &buflen);
+  res = shishi_encrypt (handle, key, keyusage, der, derlen, &buf, &buflen);
+
+  free (der);
+
   if (res != SHISHI_OK)
     {
-      shishi_error_printf (handle, "des_encrypt fail\n");
+      shishi_error_printf (handle, "Cannot encrypt EncKDCRepPart\n");
       return res;
     }
 
   res = shishi_kdcrep_set_enc_part (handle, kdcrep, shishi_key_type (key),
 				    shishi_key_version (key), buf, buflen);
 
+  free (buf);
+
   return res;
 }
 
@@ -720,8 +697,8 @@ shishi_kdcrep_decrypt (Shishi * handle,
 {
   int res;
   int i;
-  int buflen = BUFSIZ;
-  unsigned char buf[BUFSIZ];
+  char *buf;
+  size_t buflen;
   unsigned char cipher[BUFSIZ];
   int cipherlen;
   int etype;
@@ -740,13 +717,11 @@ shishi_kdcrep_decrypt (Shishi * handle,
     return res;
 
   res = shishi_decrypt (handle, key, keyusage,
-			cipher, cipherlen, buf, &buflen);
+			cipher, cipherlen, &buf, &buflen);
   if (res != SHISHI_OK)
     {
-      if (VERBOSE (handle))
-	printf ("des_decrypt failed: %s\n", shishi_strerror_details (handle));
       shishi_error_printf (handle,
-			   "des_decrypt fail, most likely wrong password\n");
+			   "KDCRep decryption failed, wrong password?\n");
       return res;
     }
 
@@ -773,6 +748,8 @@ shishi_kdcrep_decrypt (Shishi * handle,
 	break;
     }
 
+  free (buf);
+
   if (*enckdcreppart == NULL)
     {
       shishi_error_printf (handle, "Could not DER decode EncKDCRepPart. "
