@@ -123,20 +123,17 @@ _shishi_kdcreq (Shishi * handle, int as)
   if (res != SHISHI_OK)
     goto error;
 
-  res = shishi_asn1_write (handle, node, "req-body.addresses",
-			   NULL, 0);
+  res = shishi_asn1_write (handle, node, "req-body.addresses", NULL, 0);
   if (res != SHISHI_OK)
     goto error;
 
   res = shishi_asn1_write (handle, node,
-			   "req-body.enc-authorization-data", NULL,
-			   0);
+			   "req-body.enc-authorization-data", NULL, 0);
   if (res != SHISHI_OK)
     goto error;
 
   res =
-    shishi_asn1_write (handle, node, "req-body.additional-tickets",
-		       NULL, 0);
+    shishi_asn1_write (handle, node, "req-body.additional-tickets", NULL, 0);
   if (res != SHISHI_OK)
     goto error;
 
@@ -338,12 +335,11 @@ shishi_kdcreq_from_file (Shishi * handle, Shishi_asn1 * kdcreq,
 }
 
 int
-shishi_kdcreq_nonce (Shishi * handle, Shishi_asn1 kdcreq,
-		     unsigned long *nonce)
+shishi_kdcreq_nonce (Shishi * handle, Shishi_asn1 kdcreq, uint32_t * nonce)
 {
   int res;
 
-  res = shishi_asn1_integer_field (handle, kdcreq, nonce, "req-body.nonce");
+  res = shishi_asn1_read_uint32 (handle, kdcreq, "req-body.nonce", nonce);
   if (res != SHISHI_OK)
     return res;
 
@@ -381,8 +377,7 @@ shishi_kdcreq_cname_get (Shishi * handle,
 			 Shishi_asn1 kdcreq, char *cname, size_t * cnamelen)
 {
   return shishi_principal_name_get (handle, kdcreq,
-				    "req-body.cname",
-				    cname, cnamelen);
+				    "req-body.cname", cname, cnamelen);
 }
 
 int
@@ -420,8 +415,7 @@ shishi_kdcreq_set_realm (Shishi * handle, Shishi_asn1 kdcreq,
 {
   int res;
 
-  res =
-    shishi_asn1_write (handle, kdcreq, "req-body.realm", realm, 0);
+  res = shishi_asn1_write (handle, kdcreq, "req-body.realm", realm, 0);
   if (res != SHISHI_OK)
     return res;
 
@@ -433,8 +427,7 @@ shishi_kdcreq_sname_get (Shishi * handle,
 			 Shishi_asn1 kdcreq, char *sname, size_t * snamelen)
 {
   return shishi_principal_name_get (handle, kdcreq,
-				    "req-body.sname",
-				    sname, snamelen);
+				    "req-body.sname", sname, snamelen);
 }
 
 int
@@ -519,13 +512,13 @@ shishi_kdcreq_set_realmserver (Shishi * handle,
  **/
 int
 shishi_kdcreq_etype (Shishi * handle,
-		     Shishi_asn1 kdcreq, int *etype, int netype)
+		     Shishi_asn1 kdcreq, int32_t * etype, int netype)
 {
-  char buf[BUFSIZ];
+  char *buf;
   int res;
 
-  sprintf (buf, "req-body.etype.?%d", netype);
-  res = shishi_asn1_integer_field (handle, kdcreq, etype, buf);
+  asprintf (&buf, "req-body.etype.?%d", netype);
+  res = shishi_asn1_read_int32 (handle, kdcreq, buf, etype);
   if (res != SHISHI_OK)
     return res;
 
@@ -546,11 +539,10 @@ shishi_kdcreq_etype (Shishi * handle,
  **/
 int
 shishi_kdcreq_set_etype (Shishi * handle,
-			 Shishi_asn1 kdcreq, int *etype, int netype)
+			 Shishi_asn1 kdcreq, int32_t * etype, int netype)
 {
   int res;
-  char buf[BUFSIZ];
-  char buf2[BUFSIZ];
+  char *buf;
   int i;
 
   res = shishi_asn1_write (handle, kdcreq, "req-body.etype", NULL, 0);
@@ -559,14 +551,13 @@ shishi_kdcreq_set_etype (Shishi * handle,
 
   for (i = 1; i <= netype; i++)
     {
-      res = shishi_asn1_write (handle, kdcreq,
-			       "req-body.etype", "NEW", 1);
+      res = shishi_asn1_write (handle, kdcreq, "req-body.etype", "NEW", 1);
       if (res != SHISHI_OK)
 	return res;
 
-      sprintf (buf, "req-body.etype.?%d", i);
-      sprintf (buf2, "%d", etype[i - 1]);
-      res = shishi_asn1_write (handle, kdcreq, buf, buf2, 0);
+      asprintf (&buf, "req-body.etype.?%d", i);
+      res = shishi_asn1_write_int32 (handle, kdcreq, buf, etype[i - 1]);
+      free (buf);
       if (res != SHISHI_OK)
 	return res;
     }
@@ -674,7 +665,7 @@ shishi_kdcreq_add_padata_tgs (Shishi * handle,
 
   res = shishi_kdcreq_add_padata (handle, kdcreq,
 				  SHISHI_PA_TGS_REQ, data, datalen);
-  free(data);
+  free (data);
   if (res != SHISHI_OK)
     return res;
 
