@@ -94,11 +94,10 @@ shishi_sendrecv_udp (Shishi * handle,
   return SHISHI_OK;
 }
 
-int
-shishi_kdc_sendrecv (Shishi * handle,
-		     char *realm,
-		     const char *indata, size_t inlen,
-		     char *outdata, size_t * outlen)
+static int
+shishi_kdc_sendrecv_static (Shishi * handle, char *realm,
+			    const char *indata, size_t inlen,
+			    char *outdata, size_t * outlen)
 {
   int i, j, k;
   int rc;
@@ -130,6 +129,107 @@ shishi_kdc_sendrecv (Shishi * handle,
 	return SHISHI_KDC_TIMEOUT;
       }
 
-  shishi_error_printf (handle, "No KDC defined for realm %s\n", realm);
+  shishi_error_printf (handle, "No KDC defined for realm %s", realm);
   return SHISHI_KDC_NOT_KNOWN_FOR_REALM;
 }
+
+static int
+shishi_kdc_sendrecv_srv_1 (Shishi * handle, char *realm,
+			   const char *indata, size_t inlen,
+			   char *outdata, size_t * outlen,
+			   dnshost_t rrs)
+{
+  int rc;
+
+  for (; rrs; rrs = rrs->next)
+    {
+      dns_srv_t srv = (dns_srv_t) rrs->rr;
+      struct addrinfo hints;
+      struct addrinfo *ai;
+      char *port;
+
+      if (rrs->class != C_IN)
+	continue;
+      if (rrs->type != T_SRV)
+	continue;
+
+      if (VERBOSE (handle))
+	printf ("Located SRV RRs server %s:%d...\n", srv->name, srv->port);
+
+      memset (&hints, 0, sizeof(hints));
+      hints.ai_socktype = SOCK_DGRAM;
+      asprintf (&port, "%d", srv->port);
+      rc = getaddrinfo (srv->name, port, &hints, &ai);
+      free (port);
+
+      if (rc != 0)
+	{
+	  shishi_warn (handle, "Unknown KDC host `%s' (gai rc %d)",
+		       srv->name, rc);
+	  freeaddrinfo (ai);
+	  continue;
+	}
+
+      if (VERBOSE (handle))
+	printf ("Sending to %s:%d (%s)...\n", srv->name, srv->port,
+		inet_ntoa (((struct sockaddr_in *)ai->ai_addr)->sin_addr));
+
+      rc = shishi_sendrecv_udp (handle, ai->ai_addr,
+				indata, inlen, outdata, outlen,
+				handle->kdctimeout);
+
+      freeaddrinfo (ai);
+
+      if (rc != SHISHI_KDC_TIMEOUT)
+	return rc;
+    }
+
+  return SHISHI_KDC_TIMEOUT;
+}
+
+static int
+shishi_kdc_sendrecv_srv (Shishi * handle, char *realm,
+			 const char *indata, size_t inlen,
+			 char *outdata, size_t * outlen)
+{
+  dnshost_t rrs;
+  char *tmp;
+  int rc;
+
+  if (VERBOSE (handle))
+    printf ("Finding SRV RRs for %s...\n", realm);
+
+  asprintf (&tmp, "_kerberos._udp.%s", realm);
+  rrs = _shishi_resolv(tmp, T_SRV);
+  free (tmp);
+
+  if (rrs)
+    rc = shishi_kdc_sendrecv_srv_1 (handle, realm, indata, inlen,
+				    outdata, outlen, rrs);
+  else
+    {
+      shishi_error_printf (handle, "No KDC SRV RRs for realm %s\n", realm);
+      rc = SHISHI_KDC_NOT_KNOWN_FOR_REALM;
+    }
+
+  _shishi_resolv_free (rrs);
+
+  return rc;
+}
+
+int
+shishi_kdc_sendrecv (Shishi * handle, char *realm,
+		     const char *indata, size_t inlen,
+		     char *outdata, size_t * outlen)
+{
+  int rc;
+
+  rc = shishi_kdc_sendrecv_static (handle, realm,
+				   indata, inlen, outdata, outlen);
+
+  if (rc == SHISHI_KDC_TIMEOUT || rc == SHISHI_KDC_NOT_KNOWN_FOR_REALM)
+    rc = shishi_kdc_sendrecv_srv (handle, realm,
+				  indata, inlen, outdata, outlen);
+
+  return rc;
+}
