@@ -22,10 +22,12 @@
 #ifndef SHISHI_H
 #define SHISHI_H
 
-#include <stdio.h>		/* FILE */
 #include <stddef.h>		/* size_t */
+#include <stdio.h>		/* FILE */
+#include <time.h>		/* time_t */
+#include <shishi-int.h>		/* uint32_t */
 
-#define SHISHI_VERSION "0.0.0"
+#define SHISHI_VERSION "0.0.1"
 
 /* Error codes */
 enum
@@ -48,7 +50,7 @@ enum
   SHISHI_DERIVEDKEY_TOO_SMALL = 15,
   SHISHI_KEY_TOO_LARGE = 16,
   SHISHI_CRYPTO_ERROR = 17,
-  SHISHI_GCRYPT_ERROR = 18,
+  SHISHI_CRYPTO_INTERNAL_ERROR = 18,
   SHISHI_SOCKET_ERROR = 19,
   SHISHI_BIND_ERROR = 20,
   SHISHI_SENDTO_ERROR = 21,
@@ -68,7 +70,11 @@ enum
   SHISHI_OUT_OF_RANGE = 35,
   SHISHI_ASN1_NO_ELEMENT = 36,
   SHISHI_SAFE_BAD_KEYTYPE = 37,
-  SHISHI_SAFE_VERIFY_FAILED = 38
+  SHISHI_SAFE_VERIFY_FAILED = 38,
+  SHISHI_PKCS5_INVALID_PRF = 39,
+  SHISHI_PKCS5_INVALID_ITERATION_COUNT = 40,
+  SHISHI_PKCS5_INVALID_DERIVED_KEY_LENGTH = 41,
+  SHISHI_PKCS5_DERIVED_KEY_TOO_LONG = 42
 };
 
 /* This is not specified in the ASN.1 syntax for some reason. */
@@ -195,6 +201,7 @@ typedef enum
   SHISHI_RSA_MD4_DES_K = 6,
   SHISHI_RSA_MD5 = 7,
   SHISHI_RSA_MD5_DES = 8,
+  SHISHI_RSA_MD5_DES_GSS = 9,  /* XXX */
   SHISHI_HMAC_SHA1_96_AES128 = 10,
   SHISHI_HMAC_SHA1_96_AES256 = 11,
   SHISHI_HMAC_SHA1_DES3_KD = 12
@@ -284,13 +291,13 @@ Shishi_tkts_hintflags;
 struct Shishi_tkts_hint
 {
   int startpos;
-  const char *server;
-  const char *serverrealm;
-  const char *client;
-  const char *clientrealm;
+  char *server;
+  char *serverrealm;
+  char *client;
+  char *clientrealm;
   int flags;
-  int etype;
-  const char *passwd;
+  int32_t etype;
+  char *passwd;
 };
 typedef struct Shishi_tkts_hint Shishi_tkts_hint;
 
@@ -317,6 +324,7 @@ extern int shishi_init_with_paths (Shishi ** handle,
 extern int shishi_init_server (Shishi ** handle);
 extern int shishi_init_server_with_paths (Shishi ** handle,
 					  const char *systemcfgfile);
+extern void shishi_info (Shishi * handle, const char *fmt, ...);
 extern void shishi_warn (Shishi * handle, const char *fmt, ...);
 
 /* done.c */
@@ -327,8 +335,9 @@ extern int shishi_cfg (Shishi * handle, char *option);
 extern int shishi_cfg_from_file (Shishi * handle, const char *cfg);
 extern int shishi_cfg_print (Shishi * handle, FILE * fh);
 extern const char *shishi_cfg_default_systemfile (Shishi * handle);
+extern const char *shishi_cfg_default_userdirectory (Shishi * handle);
 extern const char *shishi_cfg_default_userfile (Shishi * handle);
-extern int shishi_cfg_clientkdcetype (Shishi * handle, int **etypes);
+extern int shishi_cfg_clientkdcetype (Shishi * handle, int32_t ** etypes);
 extern int shishi_cfg_clientkdcetype_set (Shishi * handle, char *value);
 
 /* error.c */
@@ -369,13 +378,12 @@ extern int shishi_principal_name_set (Shishi * handle,
 				      const char *name[]);
 extern int shishi_principal_set (Shishi * handle,
 				 Shishi_asn1 namenode,
-				 const char *namefield,
-				 const char *name);
+				 const char *namefield, const char *name);
 
 /* ticket.c */
 extern int shishi_ticket_sname_get (Shishi * handle,
 				    Shishi_asn1 ticket,
-				    char *server, int *serverlen);
+				    char *server, size_t * serverlen);
 extern int shishi_ticket_sname_set (Shishi * handle,
 				    Shishi_asn1 ticket,
 				    Shishi_name_type name_type,
@@ -393,10 +401,11 @@ extern int shishi_ticket_realm_get (Shishi * handle, Shishi_asn1 ticket,
 extern int shishi_ticket_realm_set (Shishi * handle, Shishi_asn1 ticket,
 				    const char *realm);
 extern int shishi_ticket_get_enc_part_etype (Shishi * handle,
-					     Shishi_asn1 ticket, int *etype);
+					     Shishi_asn1 ticket,
+					     int32_t * etype);
 extern int shishi_ticket_set_enc_part (Shishi * handle, Shishi_asn1 ticket,
-				       int etype, int kvno, char *buf,
-				       int buflen);
+				       int etype, int kvno,
+				       char *buf, size_t buflen);
 extern int shishi_ticket_add_enc_part (Shishi * handle, Shishi_asn1 ticket,
 				       Shishi_key * key,
 				       Shishi_asn1 encticketpart);
@@ -451,8 +460,8 @@ extern time_t shishi_tkt_endctime (Shishi_tkt * tkt);
 extern int shishi_tkt_renew_till (Shishi_tkt * tkt,
 				  char *renewtilltime, int *renewtilllen);
 extern time_t shishi_tkt_renew_tillc (Shishi_tkt * tkt);
-extern int shishi_tkt_keytype (Shishi_tkt * tkt, int *etype);
-extern int shishi_tkt_keytype_p (Shishi_tkt * tkt, int etype);
+extern int shishi_tkt_keytype (Shishi_tkt * tkt, int32_t * etype);
+extern int shishi_tkt_keytype_p (Shishi_tkt * tkt, int32_t etype);
 extern int shishi_tkt_server (Shishi_tkt * tkt,
 			      char *service, int *servicelen);
 extern int shishi_tkt_server_p (Shishi_tkt * tkt, const char *service);
@@ -494,8 +503,7 @@ extern int shishi_tkts_new (Shishi_tkts * tkts,
 extern int shishi_tkts_remove (Shishi_tkts * tkts, int ticketno);
 extern int shishi_tkts_expire (Shishi_tkts * tkts);
 extern int shishi_tkts_print_for_service (Shishi_tkts * tkts,
-					  FILE * fh,
-					  const char *service);
+					  FILE * fh, const char *service);
 extern int shishi_tkts_print (Shishi_tkts * tkts, FILE * fh);
 extern int shishi_tkts_write (Shishi_tkts * tkts, FILE * fh);
 extern int shishi_tkts_to_file (Shishi_tkts * tkts, const char *filename);
@@ -584,12 +592,12 @@ extern int shishi_authenticator_remove_cksum (Shishi * handle,
 					      Shishi_asn1 authenticator);
 extern int shishi_authenticator_cksum (Shishi * handle,
 				       Shishi_asn1 authenticator,
-				       int *cksumtype,
+				       int32_t * cksumtype,
 				       char *cksum, size_t * cksumlen);
 extern int shishi_authenticator_set_cksum (Shishi * handle,
 					   Shishi_asn1 authenticator,
 					   int cksumtype,
-					   char *cksum, int cksumlen);
+					   char *cksum, size_t cksumlen);
 extern int shishi_authenticator_add_cksum (Shishi * handle,
 					   Shishi_asn1 authenticator,
 					   Shishi_key * key,
@@ -628,12 +636,12 @@ extern int shishi_authenticator_save (Shishi * handle, FILE * fh,
 extern Shishi_asn1 shishi_as_req (Shishi_as * as);
 extern void shishi_as_req_set (Shishi_as * as, Shishi_asn1 asreq);
 extern int shishi_as_req_der (Shishi_as * as, char *out, int *outlen);
-extern int shishi_as_req_der_set (Shishi_as * as, char *der, int derlen);
+extern int shishi_as_req_der_set (Shishi_as * as, char *der, size_t derlen);
 extern Shishi_asn1 shishi_as_rep (Shishi_as * as);
 extern void shishi_as_rep_set (Shishi_as * as, Shishi_asn1 asrep);
 extern int shishi_as_rep_build (Shishi_as * as, Shishi_key * key);
 extern int shishi_as_rep_der (Shishi_as * as, char *out, int *outlen);
-extern int shishi_as_rep_der_set (Shishi_as * as, char *der, int derlen);
+extern int shishi_as_rep_der_set (Shishi_as * as, char *der, size_t derlen);
 extern Shishi_asn1 shishi_as_encasreppart (Shishi_as * as);
 extern void shishi_as_encasreppart_set (Shishi_as * as,
 					Shishi_asn1 encasreppart);
@@ -705,7 +713,7 @@ extern int shishi_asreq_cnamerealm_get (Shishi * handle, Shishi_asn1 kdcreq,
 					char *cnamerealm,
 					size_t * cnamerealmlen);
 extern int shishi_kdcreq_nonce (Shishi * handle, Shishi_asn1 kdcreq,
-				unsigned long *nonce);
+				uint32_t * nonce);
 extern int shishi_kdcreq_cname_get (Shishi * handle,
 				    Shishi_asn1 kdcreq,
 				    char *cname, size_t * cnamelen);
@@ -729,9 +737,9 @@ extern int shishi_kdcreq_set_server (Shishi * handle, Shishi_asn1 req,
 extern int shishi_kdcreq_set_realmserver (Shishi * handle, Shishi_asn1 req,
 					  char *realm, char *service);
 extern int shishi_kdcreq_etype (Shishi * handle, Shishi_asn1 kdcreq,
-				int *etype, int netype);
+				int32_t * etype, int netype);
 extern int shishi_kdcreq_set_etype (Shishi * handle, Shishi_asn1 kdcreq,
-				    int *etype, int netype);
+				    int32_t * etype, int netype);
 extern int shishi_kdcreq_clear_padata (Shishi * handle, Shishi_asn1 kdcreq);
 extern int shishi_kdcreq_add_padata (Shishi * handle,
 				     Shishi_asn1 kdcreq,
@@ -749,7 +757,7 @@ extern int shishi_enckdcreppart_srealmserver_set (Shishi * handle,
 						  const char *server);
 extern int shishi_enckdcreppart_nonce_set (Shishi * handle,
 					   Shishi_asn1 enckdcreppart,
-					   unsigned long nonce);
+					   uint32_t nonce);
 extern int
 shishi_enckdcreppart_populate_encticketpart (Shishi * handle,
 					     Shishi_asn1 enckdcreppart,
@@ -762,7 +770,7 @@ extern int shishi_enckdcreppart_flags_set (Shishi * handle,
 extern int shishi_as_derive_salt (Shishi * handle,
 				  Shishi_asn1 asreq,
 				  Shishi_asn1 asrep, char *salt,
-				  int *saltlen);
+				  size_t * saltlen);
 extern int shishi_tgs_process (Shishi * handle, Shishi_asn1 tgsreq,
 			       Shishi_asn1 tgsrep,
 			       Shishi_asn1 oldenckdcreppart,
@@ -806,7 +814,8 @@ extern int shishi_kdcrep_from_file (Shishi * handle, Shishi_asn1 * kdcrep,
 				    int filetype, char *filename);
 extern int shishi_kdcrep_clear_padata (Shishi * handle, Shishi_asn1 kdcrep);
 extern int shishi_kdcrep_get_enc_part_etype (Shishi * handle,
-					     Shishi_asn1 asrep, int *etype);
+					     Shishi_asn1 asrep,
+					     int32_t * etype);
 extern int shishi_kdcrep_add_enc_part (Shishi * handle,
 				       Shishi_asn1 kdcrep,
 				       Shishi_key * key,
@@ -822,7 +831,7 @@ extern int shishi_kdcrep_crealm_set (Shishi * handle,
 extern int shishi_kdcrep_cname_set (Shishi * handle,
 				    Shishi_asn1 kdcrep,
 				    Shishi_name_type name_type,
-				    char *cname[]);
+				    const char *cname[]);
 extern int shishi_kdcrep_client_set (Shishi * handle, Shishi_asn1 kdcrep,
 				     const char *client);
 extern int shishi_kdcrep_crealmserver_set (Shishi * handle,
@@ -880,68 +889,79 @@ extern int shishi_cipher_minpadsize (int type);
 extern int shishi_cipher_confoundersize (int type);
 extern size_t shishi_cipher_keylen (int type);
 extern size_t shishi_cipher_randomlen (int type);
-extern int shishi_cipher_defaultcksumtype (int type);
+extern int shishi_cipher_defaultcksumtype (int32_t type);
 extern int shishi_cipher_parse (const char *cipher);
+extern int shishi_checksum_supported_p (int32_t type);
+extern const char *shishi_checksum_name (int32_t type);
+extern size_t shishi_checksum_cksumlen (int32_t type);
+extern int shishi_checksum_parse (const char *checksum);
 extern int shishi_string_to_key (Shishi * handle,
-				 int keytype,
-				 const char *password,
-				 int passwordlen,
-				 const char *salt,
-				 int saltlen,
+				 int32_t keytype,
+				 const char *password, size_t passwordlen,
+				 const char *salt, size_t saltlen,
 				 const char *parameter, Shishi_key * outkey);
 extern int shishi_random_to_key (Shishi * handle,
-				 int keytype,
-				 char *random,
-				 int randomlen, Shishi_key * outkey);
+				 int32_t keytype,
+				 char *random, size_t randomlen,
+				 Shishi_key * outkey);
 extern int shishi_encrypt (Shishi * handle,
 			   Shishi_key * key,
 			   int keyusage,
-			   char *in, int inlen, char *out, int *outlen);
+			   char *in, size_t inlen,
+			   char **out, size_t * outlen);
 extern int shishi_encrypt_iv (Shishi * handle,
 			      Shishi_key * key,
 			      int keyusage,
-			      char *iv, int ivlen,
-			      char *in, int inlen,
-			      char *out, int *outlen);
+			      char *iv, size_t ivlen,
+			      char *in, size_t inlen,
+			      char **out, size_t * outlen);
 extern int shishi_encrypt_iv_etype (Shishi * handle,
 				    Shishi_key * key,
 				    int keyusage,
-				    int etype,
-				    char *iv, int ivlen,
-				    char *in, int inlen,
-				    char *out, int *outlen);
+				    int32_t etype,
+				    char *iv, size_t ivlen,
+				    char *in, size_t inlen,
+				    char **out, size_t * outlen);
 extern int shishi_decrypt (Shishi * handle,
 			   Shishi_key * key,
 			   int keyusage,
-			   char *in, int inlen, char *out, int *outlen);
+			   char *in, size_t inlen,
+			   char **out, size_t * outlen);
 extern int shishi_decrypt_iv (Shishi * handle,
 			      Shishi_key * key,
 			      int keyusage,
-			      char *iv, int ivlen,
-			      char *in, int inlen,
-			      char *out, int *outlen);
+			      char *iv, size_t ivlen,
+			      char *in, size_t inlen,
+			      char **out, size_t * outlen);
 extern int shishi_decrypt_iv_etype (Shishi * handle,
 				    Shishi_key * key,
 				    int keyusage,
-				    int etype,
-				    char *iv, int ivlen,
-				    char *in, int inlen,
-				    char *out, int *outlen);
+				    int32_t etype,
+				    char *iv, size_t ivlen,
+				    char *in, size_t inlen,
+				    char **out, size_t * outlen);
 extern int shishi_checksum (Shishi * handle,
 			    Shishi_key * key,
 			    int keyusage,
-			    int cksumtype,
-			    char *in, int inlen, char *out, int *outlen);
-extern int shishi_randomize (Shishi * handle, char *data, int datalen);
+			    int32_t cksumtype,
+			    char *in, size_t inlen,
+			    char **out, size_t * outlen);
+extern int shishi_randomize (Shishi * handle, char *data, size_t datalen);
 extern int shishi_dk (Shishi * handle,
 		      Shishi_key * key,
-		      char *constant,
-		      int constantlen, Shishi_key * derivedkey);
+		      char *constant, int constantlen,
+		      Shishi_key * derivedkey);
 extern int shishi_dr (Shishi * handle,
 		      Shishi_key * key,
-		      char *derivedkey,
-		      int derivedkeylen, char *constant, int constantlen);
-extern int shishi_n_fold (Shishi * handle, char *in, int m, char *out, int n);
+		      char *derivedkey, size_t derivedkeylen,
+		      char *constant, size_t constantlen);
+extern int shishi_n_fold (Shishi * handle, char *in, size_t m, char *out,
+			  size_t n);
+extern int shishi_pbkdf2_sha1 (const char *P, size_t Plen,
+			       const char *S, size_t Slen,
+			       unsigned int c,
+			       unsigned int dkLen,
+			       char *DK);
 
 /* version.c */
 extern const char *shishi_check_version (const char *req_version);
@@ -969,11 +989,21 @@ extern int shishi_new_a2d (Shishi * handle, Shishi_asn1 node,
 extern int shishi_asn1_done (Shishi * handle, Shishi_asn1 node);
 extern int shishi_asn1_read (Shishi * handle, Shishi_asn1 node,
 			     const char *field, char *data, size_t * datalen);
-extern int shishi_asn1_read_integer (Shishi * handle, Shishi_asn1 node,
-				     const char *field, int *i);
+extern int shishi_asn1_read_int32 (Shishi * handle, Shishi_asn1 node,
+				   const char *field, int32_t * i);
+extern int shishi_asn1_read_uint32 (Shishi * handle, Shishi_asn1 node,
+				    const char *field, uint32_t * i);
+extern int shishi_asn1_read_bitstring (Shishi * handle, Shishi_asn1 node,
+				       const char *field, int *flags);
 extern int shishi_asn1_write (Shishi * handle, Shishi_asn1 node,
 			      const char *field,
 			      const char *data, size_t datalen);
+extern int shishi_asn1_write_int32 (Shishi * handle, Shishi_asn1 node,
+				    const char *field, int32_t n);
+extern int shishi_asn1_write_uint32 (Shishi * handle, Shishi_asn1 node,
+				     const char *field, uint32_t n);
+extern int shishi_asn1_write_bitstring (Shishi * handle, Shishi_asn1 node,
+					const char *field, int flags);
 extern int shishi_asn1_field (Shishi * handle,
 			      Shishi_asn1 node,
 			      char *data, size_t * datalen,
@@ -982,9 +1012,6 @@ extern int shishi_asn1_optional_field (Shishi * handle,
 				       Shishi_asn1 node,
 				       char *data, size_t * datalen,
 				       const char *field);
-extern int shishi_asn1_integer_field (Shishi * handle,
-				      Shishi_asn1 node, int *i,
-				      const char *field);
 extern int shishi_asn1_number_of_elements (Shishi * handle,
 					   Shishi_asn1 node,
 					   const char *field, int *n);
@@ -1097,7 +1124,7 @@ extern void shishi_ap_authenticator_set (Shishi_ap * ap,
 extern Shishi_asn1 shishi_ap_req (Shishi_ap * ap);
 extern void shishi_ap_req_set (Shishi_ap * ap, Shishi_asn1 apreq);
 extern int shishi_ap_req_der (Shishi_ap * ap, char *out, int *outlen);
-extern int shishi_ap_req_der_set (Shishi_ap * ap, char *der, int derlen);
+extern int shishi_ap_req_der_set (Shishi_ap * ap, char *der, size_t derlen);
 extern int shishi_ap_req_der_new (Shishi_ap * ap, char **out, int *outlen);
 extern int shishi_ap_req_build (Shishi_ap * ap);
 extern int shishi_ap_req_asn1 (Shishi_ap * ap, Shishi_asn1 * apreq);
@@ -1106,10 +1133,11 @@ extern int shishi_ap_req_build (Shishi_ap * ap);
 
 extern Shishi_asn1 shishi_ap_rep (Shishi_ap * ap);
 extern void shishi_ap_rep_set (Shishi_ap * ap, Shishi_asn1 aprep);
-extern int shishi_ap_rep_der (Shishi_ap * ap, char *out, int *outlen);
-extern int shishi_ap_rep_der_set (Shishi_ap * ap, char *der, int derlen);
+extern int shishi_ap_rep_der (Shishi_ap * ap, char *out, size_t * outlen);
+extern int shishi_ap_rep_der_set (Shishi_ap * ap, char *der, size_t derlen);
 extern int shishi_ap_rep_verify (Shishi_ap * ap);
-extern int shishi_ap_rep_verify_der (Shishi_ap * ap, char *der, int derlen);
+extern int shishi_ap_rep_verify_der (Shishi_ap * ap, char *der,
+				     size_t derlen);
 extern int shishi_ap_rep_verify_asn1 (Shishi_ap * ap, Shishi_asn1 aprep);
 extern int shishi_ap_rep_asn1 (Shishi_ap * ap, Shishi_asn1 * aprep);
 extern int shishi_ap_rep_build (Shishi_ap * ap);
@@ -1125,16 +1153,11 @@ extern void shishi_key_principal_set (Shishi_key * key,
 extern const char *shishi_key_realm (Shishi_key * key);
 extern void shishi_key_realm_set (Shishi_key * key, const char *realm);
 extern int shishi_key_type (Shishi_key * key);
-extern void shishi_key_type_set (Shishi_key * key, int type);
+extern void shishi_key_type_set (Shishi_key * key, int32_t type);
 extern char *shishi_key_value (Shishi_key * key);
 extern void shishi_key_value_set (Shishi_key * key, const char *value);
 extern int shishi_key_value_set_random (Shishi_key * key,
-					char *random, int randomlen);
-extern int shishi_key_value_set_string (Shishi_key * key,
-					char *password,
-					int passwordlen,
-					char *salt,
-					int saltlen, char *parameter);
+					char *random, size_t randomlen);
 extern const char *shishi_key_name (Shishi_key * key);
 extern size_t shishi_key_length (Shishi_key * key);
 extern int shishi_key_version (Shishi_key * key);
@@ -1145,22 +1168,23 @@ extern void shishi_key_copy (Shishi_key * dstkey, Shishi_key * srckey);
 extern int shishi_key_print (Shishi * handle, FILE * fh, Shishi_key * key);
 extern int shishi_key_to_file (Shishi * handle,
 			       const char *filename, Shishi_key * key);
-extern int shishi_key_parse (Shishi * handle, FILE * fh, Shishi_key ** key);;
-extern int shishi_key_random (Shishi * handle, int type, Shishi_key ** key);
+extern int shishi_key_parse (Shishi * handle, FILE * fh, Shishi_key ** key);
+extern int shishi_key_random (Shishi * handle,
+			      int32_t type, Shishi_key ** key);
 extern int shishi_key_from_value (Shishi * handle,
-				  int type, char *value, Shishi_key ** key);
+				  int32_t type,
+				  char *value, Shishi_key ** key);
 extern int shishi_key_from_base64 (Shishi * handle,
-				   int type, char *value, Shishi_key ** key);
+				   int32_t type,
+				   char *value, Shishi_key ** key);
 extern int shishi_key_from_random (Shishi * handle,
-				   int type,
-				   char *random,
-				   int randomlen, Shishi_key ** outkey);
+				   int32_t type,
+				   char *random, size_t randomlen,
+				   Shishi_key ** outkey);
 extern int shishi_key_from_string (Shishi * handle,
-				   int type,
-				   const char *password,
-				   int passwordlen,
-				   const char *salt,
-				   int saltlen,
+				   int32_t type,
+				   const char *password, size_t passwordlen,
+				   const char *salt, size_t saltlen,
 				   const char *parameter,
 				   Shishi_key ** outkey);
 
@@ -1222,11 +1246,12 @@ extern int shishi_encapreppart_from_file (Shishi * handle,
 					  int filetype, char *filename);
 extern int shishi_encapreppart_get_key (Shishi * handle,
 					Shishi_asn1 encapreppart,
-					int *keytype, unsigned char *keyvalue,
-					int *keyvalue_len);
+					int32_t * keytype,
+					char *keyvalue,
+					size_t * keyvalue_len);
 extern int shishi_encapreppart_seqnumber_get (Shishi * handle,
 					      Shishi_asn1 encapreppart,
-					      int *seqnumber);
+					      uint32_t *seqnumber);
 
 /* apreq.c */
 extern Shishi_asn1 shishi_apreq (Shishi * handle);
@@ -1247,7 +1272,8 @@ extern int shishi_apreq_options_set (Shishi * handle, Shishi_asn1 apreq,
 extern int shishi_apreq_set_ticket (Shishi * handle, Shishi_asn1 apreq,
 				    Shishi_asn1 ticket);
 extern int shishi_apreq_set_authenticator (Shishi * handle, Shishi_asn1 apreq,
-					   int etype, char *buf, int buflen);
+					   int32_t etype,
+					   char *buf, int buflen);
 extern int shishi_apreq_add_authenticator (Shishi * handle, Shishi_asn1 apreq,
 					   Shishi_key * key, int keyusage,
 					   Shishi_asn1 authenticator);
@@ -1265,7 +1291,7 @@ extern int shishi_apreq_get_ticket (Shishi * handle, Shishi_asn1 apreq,
 				    Shishi_asn1 * ticket);
 extern int shishi_apreq_get_authenticator_etype (Shishi * handle,
 						 Shishi_asn1 apreq,
-						 int *etype);
+						 int32_t * etype);
 extern int shishi_apreq_decrypt (Shishi * handle, Shishi_asn1 apreq,
 				 Shishi_key * key, int keyusage,
 				 Shishi_asn1 * authenticator);
@@ -1316,7 +1342,8 @@ extern int shishi_aprep_enc_part_make (Shishi * handle, Shishi_asn1 aprep,
 				       Shishi_asn1 authenticator,
 				       Shishi_asn1 encticketpart);
 extern int shishi_aprep_get_enc_part_etype (Shishi * handle,
-					    Shishi_asn1 aprep, int *etype);
+					    Shishi_asn1 aprep,
+					    int32_t * etype);
 
 /* netio.c */
 extern int shishi_kdc_sendrecv (Shishi * handle,
@@ -1331,7 +1358,7 @@ extern int shishi_encticketpart_key_set (Shishi * handle,
 					 Shishi_key * key);
 extern int shishi_encticketpart_get_enc_part_etype (Shishi * handle,
 						    Shishi_asn1 encticketpart,
-						    int *etype);
+						    int32_t * etype);
 extern int shishi_encticketpart_get_key (Shishi * handle,
 					 Shishi_asn1 encticketpart,
 					 Shishi_key ** key);
@@ -1379,7 +1406,7 @@ extern Shishi_asn1 shishi_safe_safe (Shishi_safe * safe);
 extern void shishi_safe_safe_set (Shishi_safe * safe, Shishi_asn1 asn1safe);
 extern int shishi_safe_safe_der (Shishi_safe * safe, char *out, int *outlen);
 extern int shishi_safe_safe_der_set (Shishi_safe * safe,
-				     char *der, int derlen);
+				     char *der, size_t derlen);
 extern int shishi_safe_print (Shishi * handle, FILE * fh, Shishi_asn1 safe);
 extern int shishi_safe_save (Shishi * handle, FILE * fh, Shishi_asn1 safe);
 extern int shishi_safe_to_file (Shishi * handle, Shishi_asn1 safe,
@@ -1390,16 +1417,18 @@ extern int shishi_safe_from_file (Shishi * handle, Shishi_asn1 * safe,
 				  int filetype, char *filename);
 extern int shishi_safe_cksum (Shishi * handle,
 			      Shishi_asn1 safe,
-			      int *cksumtype, char *cksum, size_t * cksumlen);
+			      int32_t * cksumtype,
+			      char *cksum, size_t * cksumlen);
 extern int shishi_safe_set_cksum (Shishi * handle,
 				  Shishi_asn1 safe,
-				  int cksumtype, char *cksum, int cksumlen);
+				  int32_t cksumtype,
+				  char *cksum, size_t cksumlen);
 extern int shishi_safe_user_data (Shishi * handle,
 				  Shishi_asn1 safe,
 				  char *userdata, size_t * userdatalen);
 extern int shishi_safe_set_user_data (Shishi * handle,
 				      Shishi_asn1 safe,
-				      char *userdata, int userdatalen);
+				      char *userdata, size_t userdatalen);
 extern int shishi_safe_build (Shishi_safe * safe, Shishi_key * key);
 extern int shishi_safe_verify (Shishi_safe * safe, Shishi_key * key);
 
