@@ -255,13 +255,21 @@ shishi_tkt_encticketpart_set (Shishi_tkt * tkt, Shishi_asn1 encticketpart)
 Shishi_key *
 shishi_tkt_key (Shishi_tkt * tkt)
 {
-  if (!tkt->key)
+  if (!tkt->key && tkt->enckdcreppart)
     {
       int res;
 
       res = shishi_enckdcreppart_get_key (tkt->handle,
-					  shishi_tkt_enckdcreppart
-					  (tkt), &tkt->key);
+					  tkt->enckdcreppart, &tkt->key);
+      if (res != SHISHI_OK)
+	return NULL;
+    }
+  else if (!tkt->key && tkt->encticketpart)
+    {
+      int res;
+
+      res = shishi_encticketpart_get_key (tkt->handle,
+					  tkt->encticketpart, &tkt->key);
       if (res != SHISHI_OK)
 	return NULL;
     }
@@ -417,31 +425,8 @@ shishi_tkt_done (Shishi_tkt * tkt)
 int
 shishi_tkt_flags (Shishi_tkt * tkt, int *flags)
 {
-  unsigned char buf[4];
-  int buflen;
-  int i;
-  int res;
-
-  memset (buf, 0, sizeof (buf));
-  buflen = sizeof (buf);
-  res = shishi_asn1_field (tkt->handle, tkt->enckdcreppart,
-			   buf, &buflen, "flags");
-  if (res != SHISHI_OK)
-    return res;
-
-  *flags = 0;
-  for (i = 0; i < 4; i++)
-    {
-      *flags |= (((buf[i] >> 7) & 0x01) |
-		 ((buf[i] >> 5) & 0x02) |
-		 ((buf[i] >> 3) & 0x04) |
-		 ((buf[i] >> 1) & 0x08) |
-		 ((buf[i] << 1) & 0x10) |
-		 ((buf[i] << 3) & 0x20) |
-		 ((buf[i] << 5) & 0x40) | ((buf[i] << 7) & 0x80)) << (8 * i);
-    }
-
-  return SHISHI_OK;
+  return shishi_asn1_read_bitstring (tkt->handle, tkt->enckdcreppart,
+				     "flags", flags);
 }
 
 int
@@ -645,20 +630,20 @@ shishi_tkt_server_realm (Shishi_tkt * tkt,
 }
 
 int
-shishi_tkt_keytype (Shishi_tkt * tkt, int *etype)
+shishi_tkt_keytype (Shishi_tkt * tkt, int32_t * etype)
 {
-  return shishi_asn1_read_integer (tkt->handle, tkt->enckdcreppart,
-				   "key.keytype", etype);
+  return shishi_asn1_read_int32 (tkt->handle, tkt->enckdcreppart,
+				 "key.keytype", etype);
 }
 
 int
-shishi_tkt_keytype_p (Shishi_tkt * tkt, int etype)
+shishi_tkt_keytype_p (Shishi_tkt * tkt, int32_t etype)
 {
-  int tktetype;
+  int32_t tktetype;
   int rc;
 
-  rc = shishi_asn1_read_integer (tkt->handle, tkt->enckdcreppart,
-				 "key.keytype", &tktetype);
+  rc = shishi_asn1_read_int32 (tkt->handle, tkt->enckdcreppart,
+			       "key.keytype", &tktetype);
   if (rc != SHISHI_OK)
     return 0;
 
@@ -681,7 +666,7 @@ shishi_tkt_lastreq (Shishi_tkt * tkt,
 
   for (i = 1; i <= n; i++)
     {
-      shishi_asprintf (&format, "last-req.?%d.lr-type", i);
+      asprintf (&format, "last-req.?%d.lr-type", i);
       res = shishi_asn1_read_integer (tkt->handle, tkt->enckdcreppart,
 				      format, &tmplrtype);
       free (format);
@@ -690,7 +675,7 @@ shishi_tkt_lastreq (Shishi_tkt * tkt,
 
       if (lrtype == tmplrtype)
 	{
-	  shishi_asprintf (&format, "last-req.?%d.lr-value", i);
+	  asprintf (&format, "last-req.?%d.lr-value", i);
 	  res = shishi_asn1_read (tkt->handle, tkt->enckdcreppart,
 				  format, lrtime, lrtimelen);
 	  free (format);
@@ -786,8 +771,7 @@ int
 shishi_tkt_starttime (Shishi_tkt * tkt, char *starttime, int *starttimelen)
 {
   return shishi_asn1_optional_field (tkt->handle, tkt->enckdcreppart,
-				     starttime, starttimelen,
-				     "starttime");
+				     starttime, starttimelen, "starttime");
 }
 
 time_t
@@ -841,8 +825,7 @@ int
 shishi_tkt_renew_till (Shishi_tkt * tkt, char *renewtill, int *renewtilllen)
 {
   return shishi_asn1_optional_field (tkt->handle, tkt->enckdcreppart,
-				     renewtill, renewtilllen,
-				     "renew-till");
+				     renewtill, renewtilllen, "renew-till");
 }
 
 time_t
