@@ -44,10 +44,10 @@ server (Shishi * handle, struct arguments arg)
   if (arg.sname == NULL)
     {
       char *p;
-      shishi_asprintf (&p, "host/www");
+      asprintf (&p, "host/www");
       arg.sname = p;
       if (arg.sname == NULL)
-	die ("Could not allocate server name.");
+	error (1, 0, "Could not allocate server name.");
     }
 
   if (arg.verbose)
