@@ -136,6 +136,8 @@ extern char *_shishi_gettext (const char *str);
 #define LISTEN_DEFAULT "*:kerberos/udp, *:kerberos/tcp"
 #endif
 
+const char *program_name = PACKAGE;
+
 struct listenspec
 {
   char *str;
@@ -794,7 +796,7 @@ main (int argc, char *argv[])
   Shishi *handle;
   int rc;
 
-  openlog (PACKAGE, LOG_PERROR, LOG_AUTHPRIV);
+  openlog (PACKAGE, LOG_PERROR, LOG_DAEMON);
   memset ((void *) &arg, 0, sizeof (arg));
   argp_parse (&argp, argc, argv, ARGP_IN_ORDER, 0, &arg);
 
